/**
 * Persistence-log is an easy-use tool which supports customize configuration. 
 * It could save logs with different levels and complexity. This tool could meet 
 * user's need of log management on software development and running.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.log;

/**
 * Log data.
 * @author JACK LEE
 * @version 1.0 - build in 2009-11-02
 */
public class LogData {

	private String date = "";
	private String source = "";
	private String category = "";//Example : "common","secure","system"
	private String type = "";//Example : info,exception,error,fatal
	private String user = "";
	private String description = "";
	private String data = "";
	
	private String flag = FLAG;
	

	public static final String FLAG = "<new.log>";
	public static final String LOG_DATE_FLAG = "<log.date>=";//time
	public static final String LOG_SOURCE_FLAG  = "<log.source>=";//class,method,function
	public static final String LOG_CATEGORY_FLAG  = "<log.category>=";//system,security,app,model
	public static final String LOG_TYPE_FLAG  = "<log.type>=";//warn,warn,error,fatal,debug
	public static final String LOG_USER_FLAG  = "<log.user>=";//login_user
	public static final String LOG_DESCRIPTION_FLAG  = "<log.description>=";//what system should tell us
	public static final String LOG_DATA_FLAG  = "<log.data>=";//running data,just like username,exeception 
	public static final String LINE_SEPARATOR = "\r\n";
	
	
	
	/**
	 * LogData
	 * */
	public LogData(){}
	
	/**
	 * LogData
	 * @param String log
	 * @param boolean isSimpleFormat
	 * */
	public LogData(String log,boolean isSimpleFormat){
		if ( null == log || "".equals(log) ) {
			new LogData();
			return;
		}
		String[] s = log.split(LINE_SEPARATOR);
		for (String str : s)
			if (str.startsWith(LOG_DATE_FLAG)) {
				this.date = str.replace(LOG_DATE_FLAG, "");
			} else if (str.startsWith(LOG_SOURCE_FLAG)) {
				this.source = str.replace(LOG_SOURCE_FLAG, "");
			} else if (str.startsWith(LOG_CATEGORY_FLAG)) {
				this.category = str.replace(LOG_CATEGORY_FLAG, "");
			} else if (str.startsWith(LOG_TYPE_FLAG)) {
				this.type = str.replace(LOG_TYPE_FLAG, "");
			} else if (str.startsWith(LOG_USER_FLAG)) {
				this.user = str.replace(LOG_USER_FLAG, "");
			} else if (str.startsWith(LOG_DESCRIPTION_FLAG)) {
				this.description = str.replace(LOG_DESCRIPTION_FLAG, "");
			} else if (str.startsWith(LOG_DATA_FLAG)) {
				this.data = str.replace(LOG_DATA_FLAG, "");
			}
		
	}
	
	/**
	 * toString
	 * @return String
	 * */
	public String toMultipleLineString(){
		StringBuilder sb = new StringBuilder();
		if ( null == flag || "".equals(flag) )
			flag = FLAG;
		sb.append(flag).append(LINE_SEPARATOR)
		  .append(LOG_DATE_FLAG).append(date).append(LINE_SEPARATOR)
		  .append(LOG_SOURCE_FLAG).append(source).append(LINE_SEPARATOR)
		  .append(LOG_CATEGORY_FLAG).append(category).append(LINE_SEPARATOR)
		  .append(LOG_TYPE_FLAG).append(type).append(LINE_SEPARATOR)
		  .append(LOG_USER_FLAG).append(user).append(LINE_SEPARATOR)
		  .append(LOG_DESCRIPTION_FLAG).append(description).append(LINE_SEPARATOR)
		  .append(LOG_DATA_FLAG).append(data).append(LINE_SEPARATOR);
		return sb.toString();		
	}
	
	/**
	 * toString
	 * @return String
	 * */
	public String toSingleLineString(){
		StringBuilder sb = new StringBuilder();
		if ( null == flag || "".equals(flag) )
			flag = FLAG;
		sb.append("[").append(type).append("]")
		  .append("[").append(date).append("]")
		  .append(description).append(LINE_SEPARATOR);
		return sb.toString();		
	}
	
	/**
	 * @return the date
	 */
	public String getDate() {
		return date;
	}

	/**
	 * @param date the date to set
	 */
	public LogData setDate(String date) {
		this.date = date;
		return this;
	}

	/**
	 * @return the source
	 */
	public String getSource() {
		return source;
	}

	/**
	 * Set source of log, just like index.jsp
	 * @param source the source to set
	 */
	public LogData setSource(String source) {
		this.source = source;
		return this;
	}

	/**
	 * @return the category
	 */
	public String getCategory() {
		return category;
	}

	/**
	 * Set category of log, just like system,security.
	 * @param category the category to set
	 */
	public LogData setCategory(String category) {
		this.category = category;
		return this;
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * Set type, just like info,debug,fatal,error,warn.
	 * @param user the user to set
	 */
	public LogData setType(String type) {
		this.type = type;
		return this;
	}

	/**
	 * @return the user
	 */
	public String getUser() {
		return user;
	}

	/**
	 * Set user ,just like admin, visitor.
	 * @param user the user to set
	 */
	public LogData setUser(String user) {
		this.user = user;
		return this;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Set description
	 * @param description the description to set
	 */
	public LogData setDescription(String description) {
		this.description = description;
		return this;
	}
		
	/**
	 * @return the data
	 */
	public String getData() {
		return data;
	}
	/**
	 * Set data which is string
	 * @param data the data to set
	 */
	public LogData setData(String data) {
		this.data = data;
		return this;
	}
		
}
