/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.InputObject;
import com.nonesole.persistence2.config.bean.OutputObject;

/**
 * Parse Action node 
 * @author jack lee
 * @version 1.0
 * @date 2016-09-29
 */
public class ActionParser{

	private Logger logger = null;
	private static ActionParser parser;
	static {
		parser = new ActionParser();
	}
	private ActionParser(){
		logger = Logger.getInstance();
	}
	
	/**
	 * Get singleton object
	 * @return ActionParser
	 */
	public static ActionParser getInstance(){
		return parser;
	}
	
	/**
	 * parse node and build Action object
	 * @param node
	 * @return OSMObject
	 * @throws ParserException 
	 */
	public Action parse(Node node) throws ParserException{
		logger.debug("ActionParser:parse(Node)",null);
		Action action = null;
		if ( null != node && ConfigParser.ACTION.equals(node.getNodeName()) ) {
			action = new Action();
			InputObject input = null;
			OutputObject output = null;
			//Action
			Node actionNameNode = node.getAttributes().getNamedItem(ConfigParser.ACTION_NAME);
			action.setName(actionNameNode.getTextContent());
			logger.debug(">>>>>> Action name = " + actionNameNode,null);
			//Action
			Node transactionNode = node.getAttributes().getNamedItem(ConfigParser.ACTION_TRANSACTION);
			action.setTransaction("true".equals(transactionNode.getTextContent()));
			//ORM
			Node ormActionNode = XMLHandler.getInstance().getTheOnlyChildNode(node, ConfigParser.ACTION_ORM);
			if ( ormActionNode != null ) {
				action.setOrmAction(ormActionNode.getTextContent());
			}
			Node inputNode = XMLHandler.getInstance().getTheOnlyChildNode(node, ConfigParser.ACTION_INPUT);
			if ( inputNode != null ) {
				input = InputParser.getInstance().parse(inputNode);
				if ( input.validate() ) {
					action.setInput(input);
				}
			}
			Node outputNode = XMLHandler.getInstance().getTheOnlyChildNode(node, ConfigParser.ACTION_OUTPUT);
			if ( outputNode != null ) {
				output = OutputParser.getInstance().parse(outputNode);
				if ( output.validate() ) {
					action.setOutput(output);
				} else {
					throw new ParserException("The output configration of "+ action.getName() +" Action is wrong."); 
				}
			}
			List<Node> sqlNodes = XMLHandler.getInstance().getChildNodes(node, ConfigParser.ACTION_SQL);
			if ( !CommonTools.isNull(action.getOrmAction()) ) {
				return action;
			}
			if ( sqlNodes != null && !sqlNodes.isEmpty() ) {
				List<String> sqls = new ArrayList<String>();
				String sql = null;
				for ( Node n : sqlNodes ) {
					sql = SQLParser.getInstance().parse(n);
					if ( sql != null && !sql.isEmpty() ) {
						sqls.add(sql);
					}
				}
				if ( sqls.isEmpty() ) {
					throw new ParserException("The sql nodes of "+ node.getNodeName() +" Action are empty."); 
				}
				action.setSqls(sqls);
			} else {
				throw new ParserException("The sql nodes of "+ node.getNodeName() +" Action are empty."); 
			}
			
			if ( !action.validate() ) {
				throw new ParserException("The paramNode of OSM is empty.");
			}
		} 
		return action;
	}
	
}
