/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.bean.InputMapping;
import com.nonesole.persistence2.config.bean.InputObject;

/**
 * Parse Input node 
 * @author jack lee
 * @version 2.0
 * @date 2016-09-29
 */
public class InputParser {

	private Logger logger = null;
	private static InputParser parser;
	static {
		parser = new InputParser();
	}
	private InputParser(){
		logger = Logger.getInstance();
	}
	
	/**
	 * Get singleton object
	 * @return InputParser
	 */
	public static InputParser getInstance(){
		return parser;
	}
	
	/**
	 * parse node and build input object
	 * @param node
	 * @return OSMObject
	 * @throws ParserException 
	 */
	public InputObject parse(Node node) throws ParserException{
		logger.debug("InputParser:parse(Node)",null);
		InputObject input = null;
		if ( null != node && ConfigParser.ACTION_INPUT.equals(node.getNodeName()) ) {
			input = new InputObject();
			//property
			Node objectNode = XMLHandler.getInstance().getTheOnlyChildNode(
					node, ConfigParser.ACTION_INPUT_OBJECT);
			String objectProperty = XMLHandler.getInstance()
					.getNodeAttribute(objectNode, ConfigParser.ACTION_INPUT_OBJECT_PROPERTY);
			input.setProperty(objectProperty);
			
			logger.debug("input("+ node.getNodeName() +") property : " + objectProperty, null);
			
			List<Node> mappingNodes = XMLHandler.getInstance()
					.getChildNodes(objectNode, ConfigParser.ACTION_INPUT_OBJECT_MAPPING);
			if ( mappingNodes != null && !mappingNodes.isEmpty() ) {
				for ( Node n : mappingNodes ) {
					input.add(this.paseInputMapping(n));
				}
			}
		}
		return input;
	}
	
	/**
	 * Parse the column-field-mapping nodes of InputMapping object
	 * @param mappingNode
	 * @return
	 * @throws ParserException 
	 */
	private InputMapping paseInputMapping(Node mappingNode) throws ParserException {
		logger.debug("InputParser:paseInputMapping",null);
		InputMapping mapping = null;
		if ( mappingNode != null ) {
			Node sourceNode = XMLHandler.getInstance().getTheOnlyChildNode(
					mappingNode, ConfigParser.ACTION_INPUT_OBJECT_MAPPING_SOURCE);
			if ( sourceNode != null ) {
				NodeList nl = sourceNode.getChildNodes();
				if ( nl != null ) {
					Node tNode = null;
					mapping = new InputMapping();
					String tNodeName = null;
					String tNodeValue = null;
					for ( int i = 0 ; i < nl.getLength() ; i++ ) {
						tNode = nl.item(i);
						if ( tNode != null && tNode.getNodeName() != null) {
							tNodeName = tNode.getNodeName().toLowerCase(); 
							tNodeValue = tNode.getTextContent();
							if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_NAME.equals(tNodeName) ) {
								if( tNodeValue != null ) {
									mapping.setSourceName(tNodeValue);
								} else {
									throw new ParserException(tNode.getNodeName() + " has empty value.");
								}
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_CLASS.equals(tNodeName) ) {
								if( tNodeValue != null ) {
									mapping.setSourceClass(tNodeValue);
								} else {
									throw new ParserException(tNode.getNodeName() + " has empty value.");
								}
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_NOT_NULL.equals(tNodeName) ) {
								mapping.setNotNull("true".equals(tNodeValue)?true:false);
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_DEFAULT_VALUE.equals(tNodeName) ) {
								mapping.setSourceDefaultValue(null==tNodeValue?"":tNodeValue);
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_MIN_VALUE.equals(tNodeName) ) {
								mapping.setSourceMinValue(tNodeValue);
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_MAX_VALUE.equals(tNodeName) ) {
								mapping.setSourceMaxValue(tNodeValue);
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_MIN_LENGTH.equals(tNodeName) ) {
								mapping.setSourceMinLength(tNodeValue==null||"".equals(tNodeValue)?0:Integer.valueOf(tNodeValue).intValue());
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_MAX_LENGTH.equals(tNodeName) ) {
								mapping.setSourceMaxLength(tNodeValue==null||"".equals(tNodeValue)?Integer.MAX_VALUE:Integer.valueOf(tNodeValue).intValue());
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_VALUE_RULE.equals(tNodeName) ) {
								mapping.setSourceValueRule(tNodeValue);
							} else if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_WHERE.equals(tNodeName) ) {
								mapping.setWhere("true".equals(tNodeValue));;
							} else {
								
							}
						}
					}
				}
			}

			Node targetNode = XMLHandler.getInstance().getTheOnlyChildNode(
					mappingNode, ConfigParser.ACTION_INPUT_OBJECT_MAPPING_TARGET);
			if ( targetNode != null ) {
				NodeList nl = targetNode.getChildNodes();
				if ( nl != null ) {
					Node tNode = null;
					String tNodeName = null;
					String tNodeValue = null;
					for ( int i = 0 ; i < nl.getLength() ; i++ ) {
						tNode = nl.item(i);
						if ( tNode != null && tNode.getNodeName() != null) {
							tNodeName = tNode.getNodeName().toLowerCase(); 
							tNodeValue = tNode.getTextContent();
							if ( ConfigParser.ACTION_INPUT_OBJECT_MAPPING_TARGET_NAME.equals(tNodeName) ) {
								if( tNodeValue != null ) {
									mapping.setTargetName(tNodeValue);
								} else {
									throw new ParserException(tNode.getNodeName() + " has empty value.");
								}
							} else {
								
							}
						}
					}
				}
			}
		
			Node sqlParttNode = XMLHandler.getInstance().getTheOnlyChildNode(
					mappingNode, ConfigParser.ACTION_INPUT_OBJECT_MAPPING_SQL_PART);
			if ( sqlParttNode != null ) {
				if( sqlParttNode.getTextContent() != null ) {
					mapping.setSqlPart("true".equals(sqlParttNode.getTextContent()));
				}
			}
			
		}

		return mapping;
	}
	
}
