/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.bean.OSMObject;

/**
 * Parse configuration for object-Sql-mapping 
 * @author jack lee
 * @version 1.0
 * @date 2016-09-28
 */
public class OSMParser {

	private Logger logger = null;
	private static OSMParser parser;
	static {
		parser = new OSMParser();
	}
	private OSMParser(){
		logger = Logger.getInstance();
	}
	
	/**
	 * Get singleton object
	 * @return OSMParser
	 */
	public static OSMParser getInstance(){
		return parser;
	}
	
	/**
	 * parse node and build object-sql-mapping object
	 * @param node
	 * @return OSMObject
	 * @throws ParserException 
	 */
	public OSMObject parse(Node node) throws ParserException{
		logger.debug("OSMParser:parse(Node)",null);
		OSMObject osmn = null;
		if ( ConfigParser.OBJECT.equals(node.getNodeName()) ) {
			osmn = new OSMObject();
			NodeList nl = node.getChildNodes();
			for ( int i = 0 ; i < nl.getLength() ; i++ ) {
				this.paramNodeParser(nl.item(i), osmn);
			}
			if ( osmn.validate() ) {
				throw new ParserException("The paramNode of OSM is empty.");
			}
		}
		return osmn;
	}
	
	/**
	 * Parse the children nodes of object-sql-mapping object
	 * @param objNode
	 * @return
	 * @throws ParserException 
	 */
	private void paramNodeParser(Node paramNode,OSMObject object) throws ParserException {
		logger.debug("OSMParser:paramNodeParser(Node,OSMObject)",null);
		if ( null == object ) {
			throw new ParserException("The object of OSM is null.");
		}
		if ( null == paramNode ) {
			throw new ParserException("The paramNode of OSM is null.");
		}
		String nodeName = paramNode.getNodeName();
		String nodeValue = paramNode.getTextContent();
		if ( ConfigParser.OBJECT_TABLE.equals(nodeName) ) {
			object.setTableName(nodeValue);
		} else if ( ConfigParser.OBJECT_CLASS.equals(nodeName) ) {
			object.setClassName(nodeValue);
		} else if ( ConfigParser.OBJECT_PRIMERY_KEY.equals(nodeName) ) {
			object.setPrimaryKey(nodeValue);
		} else {
			
		}
	}
	
}
