/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.bean.OutputMapping;
import com.nonesole.persistence2.config.bean.OutputObject;

/**
 * Parser of Output node
 * @author jack lee
 * @version 1.0
 * @date 2016-09-29
 */
public class OutputParser {
	
	private Logger logger = null;
	private static OutputParser parser;
	static {
		parser = new OutputParser();
	}
	private OutputParser(){
		logger = Logger.getInstance();
	}
	
	/**
	 * Get the sigleton object of OutputParser
	 * @return OutputParser
	 */
	public static OutputParser getInstance(){
		return parser;
	}
	
	/**
	 * Parse node
	 * @param node
	 * @return OSMObject
	 * @throws ParserException 
	 */
	public OutputObject parse(Node node) throws ParserException{
		logger.debug("OutputParser:parse(Node)",null);
		OutputObject output = null;
		
		if ( null != node && ConfigParser.ACTION_OUTPUT.equals(node.getNodeName()) ) {
			output = new OutputObject();
			//property
			Node objectNode = XMLHandler.getInstance().getTheOnlyChildNode(node, ConfigParser.ACTION_OUTPUT_OBJECT);
			String objectProperty = XMLHandler.getInstance()
					.getNodeAttribute(objectNode, ConfigParser.ACTION_OUTPUT_OBJECT_PROPERTY);
			output.setProperty(objectProperty);
			
			//object
			Node objectClassNode = XMLHandler.getInstance().getTheOnlyChildNode(
					objectNode, ConfigParser.ACTION_OUTPUT_OBJECT_CLASS);
			String objectClass = null;
			if ( objectClassNode != null ) {
				objectClass = objectClassNode.getTextContent();
				output.setObjectClass(objectClass);
			}
			
			//item
			Node itemClassNode = XMLHandler.getInstance().getTheOnlyChildNode(
					objectNode, ConfigParser.ACTION_OUTPUT_OBJECT_ITEM_CLASS);
			String itemClass = null;
			if ( itemClassNode != null ) {
				itemClass = itemClassNode.getTextContent();
				output.setItemClass(itemClass);
			}
			
			//OutputMapping
			List<Node> mappingNodes = XMLHandler.getInstance()
					.getChildNodes(objectNode, ConfigParser.ACTION_OUTPUT_OBJECT_MAPPING);
			if ( mappingNodes != null && !mappingNodes.isEmpty() ) {
				for ( Node n : mappingNodes ) {
					output.add(this.paseOutputMapping(n));
				}
			}
		}
		return output;
	}
	
	/**
	 * This method will get the relation between columns of talbe and fields of object which is set in Output node.
	 * @param mappingNode
	 * @return
	 * @throws ParserException 
	 */
	private OutputMapping paseOutputMapping(Node mappingNode) throws ParserException {
		logger.debug("OutputParser:paseOutputMapping(Node)",null);
		OutputMapping mapping = null;
		if ( mappingNode != null ) {
			Node sourceNode = XMLHandler.getInstance().getTheOnlyChildNode(
					mappingNode, ConfigParser.ACTION_OUTPUT_OBJECT_MAPPING_SOURCE);
			if ( sourceNode != null ) {
				NodeList nl = sourceNode.getChildNodes();
				if ( nl != null ) {
					Node tNode = null;
					mapping = new OutputMapping();
					String tNodeName = null;
					String tNodeValue = null;
					for ( int i = 0 ; i < nl.getLength() ; i++ ) {
						tNode = nl.item(i);
						if ( tNode != null && tNode.getNodeName() != null) {
							tNodeName = tNode.getNodeName().toLowerCase(); 
							tNodeValue = tNode.getTextContent();
							if ( ConfigParser.ACTION_OUTPUT_OBJECT_MAPPING_SOURCE_NAME.equals(tNodeName) ) {
								if( tNodeValue != null ) {
									mapping.setSourceName(tNodeValue);
								} else {
									throw new ParserException(tNode.getNodeName() + " has empty value.");
								}
							} else {
								
							}
						}
					}
				}
			}

			Node targetNode = XMLHandler.getInstance().getTheOnlyChildNode(
					mappingNode, ConfigParser.ACTION_OUTPUT_OBJECT_MAPPING_TARGET);
			if ( targetNode != null ) {
				NodeList nl = targetNode.getChildNodes();
				if ( nl != null ) {
					Node tNode = null;
					String tNodeName = null;
					String tNodeValue = null;
					for ( int i = 0 ; i < nl.getLength() ; i++ ) {
						tNode = nl.item(i);
						if ( tNode != null && tNode.getNodeName() != null) {
							tNodeName = tNode.getNodeName().toLowerCase(); 
							tNodeValue = tNode.getTextContent();
							if ( ConfigParser.ACTION_OUTPUT_OBJECT_MAPPING_TARGET_NAME.equals(tNodeName) ) {
								if( tNodeValue != null ) {
									mapping.setTargetName(tNodeValue);
								} else {
									throw new ParserException(tNode.getNodeName() + " has empty value.");
								}
							} else if ( ConfigParser.ACTION_OUTPUT_OBJECT_MAPPING_TARGET_CLASS.equals(tNodeName) ) {
								if( tNodeValue != null ) {
									mapping.setTargetClass(tNodeValue);
								} else {
									throw new ParserException(tNode.getNodeName() + " has empty value.");
								}
							} else {
								
							}
						}
					}
				}
			}
		
		}
		return mapping;
	}
	
}
