/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.omg.CORBA.Environment;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * XML Tools
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-21
 */
public class XMLHandler {

	private static XMLHandler handler;
	
	static {
		handler = new XMLHandler();
	}
	
	private XMLHandler(){}
	
	/**
	 * Get XMLHandler Object
	 * @return XMLHandler Object
	 * */
	public static XMLHandler getInstance(){
		return handler;
	}
		
	/**
	 * Read XML File and return Document object
	 * @param filePath String
	 * @return document Document
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 * */
	public final Document loadFile(String filePath) 
	throws ParserConfigurationException, SAXException, IOException{
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = null;
		doc = db.parse(filePath);
		return doc;
	}
	
	/**
	 * Read XML File and return Document object
	 * @param filePath String
	 * @return document Document
	 * @throws Exception 
	 * */
	public final Document loadStreamFile(String resource) 
	throws Exception{
		InputStream is = getResourceAsStream(resource);
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = null;
		doc = db.parse(is);
		return doc;
	}
	
	/**
	 * Get inputstream from file
	 * @param resource
	 * @return InputStream
	 * @throws Exception 
	 */
	public InputStream getResourceAsStream(String resource) throws Exception {
		String stripped = resource.startsWith("/") ? resource.substring(1)
				: resource;
		InputStream stream = null;
		ClassLoader classLoader = Thread.currentThread()
				.getContextClassLoader();
		if (classLoader != null) {
			stream = classLoader.getResourceAsStream(stripped);
		}
		if (stream == null) {
			stream = Environment.class.getResourceAsStream(resource);
		}
		if (stream == null) {
			stream = Environment.class.getClassLoader().getResourceAsStream(
					stripped);
		}
		if (stream == null) {
			throw new Exception(resource + " not found");
		}
		return stream;
	}
	
	/**
	 * Read XML File and return Document object
	 * @param is InputStream
	 * @return document Document
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 * */
	public final Document loadFile(InputStream is) 
	throws ParserConfigurationException, SAXException, IOException{
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = null;
		doc = db.parse(is);
		return doc;
	}
	
	/**
	 * Transform StringBuilder to Document 
	 * @param xmlString - StringBuilder
	 * @return Document
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 */
	public final Document parse(StringBuilder xmlString) 
		throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilder builder = createDocumentBuilder();
		Reader reader = new StringReader(xmlString.toString());
		InputSource is = new InputSource(reader);
		return builder.parse(is);
	}

	/**
	 * byte[] to Document
	 * @param xml - the byte[] array object;
	 * @return Document Object;
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 */
	public final Document parse(byte[] xml) 
		throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilder builder = createDocumentBuilder();
		ByteArrayInputStream input = new ByteArrayInputStream(xml);
		return builder.parse(input);
	}
	
	/**
	 * Create new Document
	 * @return DOM Document Object;
	 * @throws ParserConfigurationException 
	 */
	public final Document createDocument() 
		throws ParserConfigurationException {
		DocumentBuilder builder = createDocumentBuilder();
		return builder.newDocument();
	}

	
	/**
	 * Craete DocumentBuilder Object
	 * @throws ParserConfigurationException 
	 * */
	private final DocumentBuilder createDocumentBuilder() 
		throws ParserConfigurationException{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		return builder;
	}
	
	/**
	 * Transform Document to String
	 * @param doc Document
	 * @return result
	 */
	public final String xmlDocumentToString(Document doc,String charactor) {
		TransformerFactory  tf  =  TransformerFactory.newInstance();
		Transformer t;
		String xmlStr = "";
		try {
			t = tf.newTransformer();
			t.setOutputProperty("encoding",charactor);
			ByteArrayOutputStream  bos  =  new  ByteArrayOutputStream();
			t.transform(new DOMSource(doc), new StreamResult(bos));
			xmlStr = bos.toString();
		} catch (TransformerConfigurationException e) {
			return "";
		} catch (TransformerException e) {
			return "";
		}
		return xmlStr;
	}
	
	/**
	 * Transform Document to StringBuilder
	 * @param doc Document
	 * @return StringBuilder
	 */
	public final StringBuilder xmlDocumentToStringBuilder(Document doc,String character) {
		return new StringBuilder(xmlDocumentToString(doc,character));
	}
	
	/**
	 * Transform Document to byte[]
	 * @param doc Document
	 * @return byte[] 
	 * */
	public final byte[] parseToStream(Document doc,String character){
		return xmlDocumentToString(doc,character).getBytes();
	}
	
	/**
	 * Get list of node whose name is given
	 * @param node father node
	 * @param nodeName child node name
	 * @return list of child nodes
	 * */
	public final List<Node> getNodes(Node node,String nodeName) {
		if ( null == node )
			return null;
		if ( null == nodeName || "".equals(nodeName) )
			return null;
		NodeList nl = node.getChildNodes();
		List<Node> r = new ArrayList<Node>();
		for ( int i = 0 ; i < nl.getLength() ; i++  ) {
			//System.out.println(nodeName.toLowerCase() + " : " + nl.item(i).getNodeName().toLowerCase());
			if ( nodeName.toLowerCase().equals(nl.item(i).getNodeName().toLowerCase()) ) {
					r.add(nl.item(i));
			}
		}
		return r;
	}	
	
	/**
	 * Get list of node whose name is given in all nodes.
	 * @param node father node
	 * @param nodeName child node name
	 * @return list of child nodes
	 * */
	public final List<Node> getChildNodes(Node node,String nodeName) {
		if ( null == node )
			return null;
		if ( null == nodeName || "".equals(nodeName) )
			return null;
		NodeList nl = node.getChildNodes();
		List<Node> r = new ArrayList<Node>();
		List<Node> cr = null;
		Node thisNode = null;
		for ( int i = 0 ; i < nl.getLength() ; i++  ) {
			thisNode = nl.item(i);
			//System.out.println(nodeName.toLowerCase() + " : " + thisNode.getNodeName().toLowerCase());
			if ( nodeName.toLowerCase().equals(thisNode.getNodeName().toLowerCase()) ) {
					r.add(nl.item(i));
			} else {
				if ( nl.item(i).hasChildNodes() ) {
					cr = this.getChildNodes(thisNode, nodeName);
					if ( cr != null && !cr.isEmpty() ) {
						r.addAll(cr);
					}
				}
			}
		}
		return r;
	}	
	
	/**
	 * Get attribute value of node.
	 * @param node 
	 * @param attrName 
	 * @return String
	 * */
	public final String getNodeAttribute(Node node,String attrName) {
		if ( null == node )
			return null;
		if ( node != null ) {
			NamedNodeMap attributesMap = node.getAttributes();
			if ( attributesMap != null ) {
				Node objectProperty = attributesMap.getNamedItem(attrName);
				if ( objectProperty != null ) {
					return objectProperty.getTextContent();
				}
			}
		}
		return null;
	}	
	
	
	/**
	 * ��ָ����Ƶĸ��ڵ�ȡ��ָ����Ƶ��ӽڵ�
	 * 
	 * Get children nodes of parent node by name 
	 * @param parentNodeName
	 * @param childNodeName
	 * @return List<Node>
	 */
	public final List<Node> getChildNodes(Document doc,String parentNodeName,String childNodeName){
		if ( null != doc ) {
			List<Node> osm = this.getChildNodes(doc, parentNodeName);
			if ( null != osm && !osm.isEmpty() ) {
				List<Node> osmNodes = new ArrayList<Node>();
				List<Node> l = null;
				for ( Node n : osm ) {
					l = XMLHandler.getInstance().getNodes(n, childNodeName);
					if ( l != null && !l.isEmpty() ) {
						osmNodes.addAll(l);
					}
				}
				return osmNodes;
			}
		}
		return null;
	}
	
	/**
	 * Action��Input��Output����Ψһ�ڵ㣬���ô˷�����鲢�õ��ýڵ㡣
	 *
	 * This method is used to get the only one node of action just like the Input node or Output node.
	 * @param parentNode
	 * @param childNodeName
	 * @return
	 * @throws ParserException
	 */
	public Node getTheOnlyChildNode(Node parentNode,String childNodeName) 
			throws ParserException{
		List<Node> inputNodes = XMLHandler.getInstance().getChildNodes(parentNode, childNodeName);
		if ( inputNodes != null && !inputNodes.isEmpty() ) {
			if ( inputNodes.size() != 1 ){
				throw new ParserException("The node of "+ parentNode.getNodeName() +" Action is empty."); 
			} else {
				return inputNodes.get(0);
			}
		}
		return null;
	}
}
