/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config.bean;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.IOrmMaker;
import com.nonesole.persistence2.config.ParserException;

/**
 * Action object
 * @author jack lee
 * @version 1.0
 */
public class Action {
	
	public static final String METHOD_INSERT = "insert";
	public static final String METHOD_UPDATE = "update";
	public static final String METHOD_DELETE = "delete";
	public static final String METHOD_SELECT = "select";
	
	private Logger logger = null;
	private InputObject input;
	private OutputObject output;
	private List<String> sqls;
	private boolean transaction = false;
	private String name = null;
	private String ormAction = null;
	private Map<String,Boolean> whereParameters = null;
	private Map<String,Class<?>> parameterClass = null;
	
	private List<String> parameters = null;
	
	public Action(){
		sqls = new ArrayList<String>();
		logger = Logger.getInstance();
	}

	public InputObject getInput() {
		return input;
	}

	public void setInput(InputObject input) {
		this.input = input;
	}

	public OutputObject getOutput() {
		return output;
	}

	public void setOutput(OutputObject output) {
		this.output = output;
	}
	
	/**
	 * @param sql
	 * */
	public void addSql(String sql){
		this.sqls.add(sql);
	}
	
	/**
	 * @return the sqls
	 */
	public List<String> getSqls() {
		return sqls;
	}

	/**
	 * @param sqls the sqls to set
	 */
	public void setSqls(List<String> sqls) {
		this.sqls = sqls;
	}

	public boolean isTransaction() {
		return transaction;
	}

	public void setTransaction(boolean transaction) {
		this.transaction = transaction;
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * map,simple
	 * @param input
	 * @return
	 * @throws ParserException
	 */
	public List<String> map2SQL(Map<String,Object> map) throws ParserException{
		logger.debug("Action:object2SQL(Map)",null);
		
		if ( this.input != null ) {
			logger.debug(">>>>>>>>> input:property: " + input.getProperty(), null);
			if ( InputObject.MAP.equals(this.input.getProperty()) ) {
				this.checkMapParam(map);
				return this.setValue2SQL(map);
			} else {
				throw new ParserException("The input object has wrong type.");
			}
		} else {
			return sqls;
		}
	}
	
	/**
	 * @param value
	 * @param type
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private Object checkNotNull(Object value, Class<?> type, InputMapping m) 
	throws ParserException {
		if ( m.isNotNull() ) {
			if ( value == null ) {
				if ( CommonTools.isNull(m.getSourceDefaultValue()) ) {
					throw new ParserException("The value and default value of "+ m.getSourceName() +" field are null, which is not follow the rules.");
				} else {
					value = m.getSourceDefaultValue();
				}
			} 
		} else {
			if ( value == null ) {
				if ( this.isNumberType(type) ) {
					value = "0";
				} else {
					value = "";
				}
			}
		}
		return value;
	}
	
	/**
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkNumber(Object value, Class<?> type, InputMapping m) throws ParserException {
		if ( this.isNumberType(type) && !CommonTools.isNumber(value.toString()) ) {
			throw new ParserException("The value of "+ m.getSourceName() +" field is not number.");
		} 
	}
	
	/**
	 * ���min-value������Ҫ�󣬼�鴫�������Ƿ���ڵ���min-value��<br/>
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkMinValue(Object value, InputMapping m) throws ParserException {
		if ( !CommonTools.isNull(m.getSourceMinValue()) ) {
			if ( !CommonTools.isNumber(m.getSourceMinValue()) ){
				throw new ParserException("The mini value of "+ m.getSourceName() +" field is wrong type.");
			} else {
				Double dValue = Double.valueOf(value.toString());
				if ( !m.greaterThanMinValue(dValue) ) {
					throw new ParserException("The value of "+ m.getSourceName() +" field is less than the min value.");
				} 
			}	
		}
	}
	
	/**
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkMaxValue(Object value, InputMapping m) throws ParserException {
		if ( !CommonTools.isNull(m.getSourceMaxValue()) ) {
			if ( !CommonTools.isNumber(m.getSourceMaxValue()) ){
				//���ֵ�����֣����쳣
				throw new ParserException("The mini value of "+ m.getSourceName() +" field is wrong type.");
			} else {
				Double dValue = Double.valueOf(value.toString());
				if ( !m.lessThanMaxValue(dValue) ) {
					//�������ֵ�����쳣
					throw new ParserException("The value of "+ m.getSourceName() +" field is greater than the max value.");
				}
			}	
		}
	}
	
	/**
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkMinLength(String value, InputMapping m) throws ParserException {
		//������ַ����ͣ�������Ƿ��г������ƣ������ֵ����Сֵ
		if ( !CommonTools.isNull(m.getSourceMinLength()) && !m.greaterThanMinLength(value) ) {
			//��������ʽ��ƥ�䣬���쳣
			throw new ParserException("The length of "+ m.getSourceName() +" field is less than the mini value.");
		}
	}
	
	/**
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkMaxLength(String value, InputMapping m) throws ParserException {
		if ( !CommonTools.isNull(m.getSourceMaxLength()) && !m.lessThanMaxLength(value) ) {
			throw new ParserException("The length of "+ m.getSourceName() +" field is greater than the max value.");
		}
	}	
	
	/**
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkValueRule(String value, InputMapping m) throws ParserException {
		if ( !CommonTools.isNull(m.getSourceValueRule()) && !m.matchValueRule(value) ) {
			throw new ParserException("The value of "+ m.getSourceName() +" field is not match the rules.");
		}
	}	
	
	/**
	 * @param value
	 * @param m
	 * @return
	 * @throws ParserException
	 */
	private void checkTargetName(InputMapping m) throws ParserException {
		if ( !this.isOrmModel() && CommonTools.isNull(m.getTargetName()) ) {
			throw new ParserException("The target name is null.");
		}
	}
	
	/**
	 * @throws ParserException
	 */
	public void checkMapParam(Map<String,Object> map) throws ParserException {
		if ( input == null ) {
			return;
		}
		if ( map == null ) {
			throw new ParserException("The map param is null.");
		}
		if ( this.input.getMappings() == null  ) {
			throw new ParserException("The input mapping is not empty.");
		} else {
			List<InputMapping> ml = this.input.getMappings();
			Object value = null;
			Class<?> fieldType = null;
			for ( InputMapping m : ml ) {
				try {
					fieldType = this.getClass(m.getSourceClass());
				} catch (ClassNotFoundException e) {
					throw new ParserException("The object class is wrong.",e.fillInStackTrace());
				}
				value = map.get(m.getSourceName());
				value = this.checkNotNull(value, fieldType, m);
				
				if ( this.isNumberType(fieldType) ) {
					this.checkNumber(value, fieldType, m);
					this.checkMinValue(value, m);
					this.checkMaxValue(value, m);
				} else {
					String sValue = value.toString();
					this.checkValueRule(sValue, m);
					this.checkMinLength(sValue, m);
					this.checkMaxLength(sValue, m);
				}
				
				this.checkTargetName(m);
			}
		}
	}
	
	/**
	 * simple-list,pojo-list
	 * @param input
	 * @return
	 * @throws ParserException
	 */
	public List<String> list2SQL(List<Object> list) throws ParserException{
		if ( list == null || list.isEmpty() ) {
			throw new ParserException("The object-list is null.");
		}
		logger.debug("Action:object2SQL(List)",null);
		if ( this.input != null ) {
			if ( InputObject.POJO_LIST.equals(this.input.getProperty()) ) {
				//���У��
				List<String> sqlList = new ArrayList<String>();
				for ( Object object : list ) {
					sqlList.addAll(this.object2SQL(object));
				}
				return sqlList;
			} else if ( this.input != null ) {
				if ( InputObject.SIMPLE_LIST.equals(this.input.getProperty()) ) {
					if ( this.input.getMappings() == null || this.input.getMappings().isEmpty() ) {
						throw new ParserException("The input mapping is null.");
					}
					if ( this.input.getMappings().size() != 1 ) {
						throw new ParserException("The input mapping is more than one object.");
					}
					List<String> sqlList = new ArrayList<String>();
					String rSql = null;
					for ( Object object : list ) {
						this.checkSimpleParam(object);
						for ( int i = 0 ; i < this.sqls.size() ; i++ ) {
							rSql = this.sqls.get(i);
							rSql = rSql.replace(
									this.input.getMappings().get(0).getTargetName(), 
									object.toString());
							sqlList.add(rSql);
						}
					}
					return sqlList;
				} else {
					throw new ParserException("The input object has wrong type.");
				}
			} else {
				throw new ParserException("The input object has wrong type.");
			}
		} else {
			return sqls;
		}
	}
	
	/**
	 * @throws ParserException
	 */
	public void checkSimpleParam(Object object) throws ParserException {
		if ( input == null ) {
			return;
		}
		if ( this.input.getMappings() == null  ) {
			throw new ParserException("The input mapping is not empty.");
		} else {
			List<InputMapping> ml = this.input.getMappings();
			Class<?> fieldType = null;
			for ( InputMapping m : ml ) {
				try {
					fieldType = this.getClass(m.getSourceClass());
				} catch (ClassNotFoundException e) {
					throw new ParserException("The object class is wrong.",e.fillInStackTrace());
				}
				
				object = this.checkNotNull(object, fieldType, m);
				
				if ( this.isNumberType(fieldType) ) {
					this.checkNumber(object, fieldType, m);
					this.checkMinValue(object, m);
					this.checkMaxValue(object, m);
				} else {
					String sValue = object.toString();
					this.checkValueRule(sValue, m);
					this.checkMinLength(sValue, m);
					this.checkMaxLength(sValue, m);
				}
				
				this.checkTargetName(m);
			}
		}
	}
	
	/**
	 * pojo
	 * @param input
	 * @return
	 * @throws ParserException
	 */
	@SuppressWarnings("unchecked")
	public List<String> object2SQL(Object object) throws ParserException{
		logger.debug("Action:object2SQL(Object)",null);
		if ( object != null ) {
			if ( object instanceof Map<?,?> ) {
				return this.map2SQL((Map<String, Object>)object);
			} else if ( object instanceof List<?> ) {
				return this.list2SQL((List<Object>)object);
			} else {
					if ( this.input != null ) {
						if ( InputObject.POJO.equals(this.input.getProperty()) ) {
							Map<String,Object> valueMap = this.checkPojoParam(object);
							return this.setValue2SQL(valueMap);
						} else {
							throw new ParserException("The input object has wrong type.");
						}
					} else {
						return sqls;
					}
			}
		} 
		return null;
	}
	
	/**
	 * orm-pojo,orm-pojo-list
	 * @param maker
	 * @return SQL
	 * @throws ParserException 
	 */
	public String orm2SQL(IOrmMaker maker,Object object) throws ParserException{
		logger.debug("Action:orm2SQL(IOrmMaker)",null);
		if ( maker != null ) {
			if ( object != null ) {
				if ( input == null ) {
					throw new ParserException("Input is null.");
				}
				if ( InputObject.ORM_POJO.equals(this.input.getProperty()) ||
						InputObject.ORM_POJO_LIST.equals(this.input.getProperty())) {
					Map<String,Object> fieldValueMap = this.checkPojoParam(object);
					try {
						return maker.makeSQL(object,fieldValueMap,this,false);
					} catch (Exception e) {
						throw new ParserException("Making sql in orm-model has error.",e.fillInStackTrace());
					}
				} else {
					throw new ParserException("This is not an orm action, please change the right one.");
				}
			} else {
				throw new ParserException("Action object is null.");
			}
		} else {
			throw new ParserException("The orm maker is null.");
		}
	}
	
	/**
	 * orm-pojo,orm-pojo-list
	 * @param maker
	 * @return SQL
	 * @throws ParserException 
	 */
	public String orm2PreparedSQL(IOrmMaker maker,Object object) throws ParserException{
		logger.debug("Action:orm2PreparedSQL(IOrmMaker)",null);
		if ( maker == null ) {
			throw new ParserException("The orm maker is null.");
		}
		if ( object == null ) {
			throw new ParserException("Action object is null.");
		}
		if ( input == null ) {
			throw new ParserException("Input is null.");
		}
		if ( InputObject.ORM_POJO.equals(this.input.getProperty()) ||
				InputObject.ORM_POJO_LIST.equals(this.input.getProperty())) {
			Map<String,Object> fieldValueMap = this.checkPojoParam(object);
			try {
				return maker.makeSQL(object,fieldValueMap,this,true);
			} catch (Exception e) {
				throw new ParserException("Making sql in orm-model has error.",e.fillInStackTrace());
			}
		} else {
			throw new ParserException("This is not an orm action, please change the right one.");
		}
		
	}
	
	/**
	 * @throws ParserException
	 */
	public Map<String,Object> checkPojoParam(Object object) throws ParserException {
		Map<String,Object> valueMap = new HashMap<String,Object>();
		if ( input == null ) {
			return valueMap;
		}
		Class<?> pojoClass = object.getClass();
		if ( this.isPrimitiveType(pojoClass) || this.isSingleValueType(pojoClass) ) {
			throw new ParserException("The input object has wrong type.");
		}
		if ( InputObject.ORM_POJO.equals(this.input.getProperty()) ) {
			if ( CommonTools.isNull(this.getOrmAction()) ) {
				throw new ParserException("The input object("+ this.name +") is orm-pojo,so the action of it is not empty.");
			} 
		}
		if ( this.input.getMappings() == null  ) {
			throw new ParserException("The input mapping is not empty.");
		} else {
			List<InputMapping> ml = this.input.getMappings();
			Object value = null;
			Class<?> fieldType = null;
			for ( InputMapping m : ml ) {
				try {
					fieldType = this.getClass(m.getSourceClass());
				} catch (ClassNotFoundException e) {
					throw new ParserException("The field type is wrong.",e.fillInStackTrace());
				}
				try {
					value = this.getFieldValue(m.getSourceName(), fieldType, object);
				} catch (Exception e) {
					throw new ParserException("The field value is wrong.",e.fillInStackTrace());
				}
				logger.debug(">>>>>>> checkPojoParam " + m.getSourceName() + " , "+value +" , "+ fieldType, null);
				//�ж�value�Ƿ�Ϊ��
				value = this.checkNotNull(value, fieldType, m);
				
				if ( this.isNumberType(fieldType) ) {
					this.checkNumber(value, fieldType, m);
					this.checkMinValue(value, m);
					this.checkMaxValue(value, m);
				} else {
					String sValue = value.toString();
					this.checkValueRule(sValue, m);
					this.checkMinLength(sValue, m);
					this.checkMaxLength(sValue, m);
				}
				
				this.checkTargetName(m);
				
				valueMap.put(m.getSourceName(), value);
			}
		}
		return valueMap;
	}
	
	/**
	 * @param valueMap
	 * @return
	 */
	private List<String> setValue2SQL(Map<String,Object> valueMap){
		logger.debug("Action:map2SQL(Map)",null);
		List<String> list = new ArrayList<String>();
		if ( input == null ) {
			return list;
		}
		String targetSql = null;
		String key = null;
		Object value = null;
		for ( String sql : this.sqls ) {
			targetSql = sql;
			for ( InputMapping map : this.input.getMappings() ) {
				key = map.getTargetName();
				value = valueMap.get(map.getSourceName());
				if ( value == null ) {
					value = "";
				}
				targetSql = targetSql.replace(key, value.toString());
			}
			list.add(targetSql);
		}
		return list;
	} 
	
	/**
	 * @param clazz
	 * @return
	 */
	private boolean isPrimitiveType(Class<?> clazz){
		return clazz == int.class || clazz == Integer.class ||
				clazz == short.class || clazz == Short.class ||
				clazz == long.class || clazz == Long.class ||
				clazz == byte.class || clazz == Byte.class ||
				clazz == char.class || clazz == Character.class ||
				clazz == float.class || clazz == Float.class ||
				clazz == double.class || clazz == Double.class ||
				clazz == boolean.class || clazz == Boolean.class;
	}
	
	/**
	 * @param clazz
	 * @return
	 */
	public boolean isNumberType(Class<?> clazz){
		return clazz == int.class || clazz == Integer.class ||
				clazz == short.class || clazz == Short.class ||
				clazz == long.class || clazz == Long.class ||
				clazz == float.class || clazz == Float.class ||
				clazz == double.class || clazz == Double.class;
	}
	
	/**
	 * @param clazz
	 * @return
	 */
	public boolean isSingleValueType(Class<?> clazz){
		return clazz == String.class;
	}
	
	/**
	 * @param clazz
	 * @return
	 * @throws ClassNotFoundException 
	 */
	public Class<?> getClass(String clazz) throws ClassNotFoundException{
		if ( clazz == null || "".equals(clazz) ) {
			return Object.class;
		} else {
			if ( "int".equals(clazz) ) {
				return int.class;
			} else if ( "long".equals(clazz) ) {
				return long.class;
			} else if ( "short".equals(clazz) ) {
				return short.class;
			} else if ( "byte".equals(clazz) ) {
				return byte.class;
			} else if ( "char".equals(clazz) ) {
				return String.class;
			} else if ( "boolean".equals(clazz) ) {
				return boolean.class;
			} else if ( "float".equals(clazz) ) {
				return Float.class;
			} else if ( "double".equals(clazz) ) {
				return double.class;
			} else if ( "String".equals(clazz) ) {
				return String.class;
			} else {
				return Class.forName(clazz);
			}
		}
	}
	
	/**
	 * Get value by get method
	 * @param field Field object
	 * @param obj owner of field
	 * @throws ParserException
	 * */
	private Object getFieldValue(String fieldName,Class<?> fieldType,Object obj) 
	throws Exception{
		Method m = null;
		Class<?> classType = obj.getClass();
		StringBuilder sb = new StringBuilder();
		if ( fieldType == boolean.class || fieldType == Boolean.class ) {
		    if ( "is".equals(fieldName.substring(0,2)) )
		    	sb.append("is").
				append(fieldName.substring(2, fieldName.length()));
		    else
				sb.append("is").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		
		} else
			sb.append("get").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		
		m = classType.getMethod(sb.toString(),new Class[]{});
		return m.invoke(obj, new Object[]{});
	}
	
	/**
	 * @return
	 */
	public boolean validate(){
		if ( CommonTools.isNull(this.name) ) {
			return false;
		}
		if ( this.sqls == null || this.sqls.isEmpty() ) {
			return false;
		}
		
		if ( input != null ) {
			if ( InputObject.ORM_POJO.equals(input.getProperty()) || 
					InputObject.ORM_POJO_LIST.equals(input.getProperty()) ) {
				if ( !CommonTools.isNull(this.ormAction) ) {
					if ( Action.METHOD_INSERT.equals(this.ormAction) ||
							Action.METHOD_UPDATE.equals(this.ormAction) ||
							Action.METHOD_DELETE.equals(this.ormAction) ||
							Action.METHOD_SELECT.equals(this.ormAction) ) {
						logger.debug("This is orm-plus action.", null);
					} else {
						logger.error("The item's name of orm-plus action is wrong.", null);
						return false;
					}
				}
			}
			if ( !this.input.validate() ) {
				return false;
			}
		}
		
		if ( this.output != null ) {
			if ( !this.output.validate() ) {
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * @return List
	 */
	public List<String> getParameters(){
		if ( parameters == null ) {
			parameters = new ArrayList<String>();
			if ( input != null && input.getMappings() != null && 
					!input.getMappings().isEmpty() ) {
				for ( InputMapping m : input.getMappings() ) {
					parameters.add(m.getSourceName());
				}
			}
		}	
		return parameters;
	}
	
	/**
	 * @param parameter
	 * @return boolean
	 */
	public boolean isWhereParameter(String parameter) {
		if ( whereParameters == null ) {
			whereParameters = new HashMap<String,Boolean>();
			if ( this.input.getMappings() != null ) {
				for ( InputMapping im : this.input.getMappings() ) {
					whereParameters.put(im.getSourceName(), im.isWhere());
				}
			}
		}
		return whereParameters.get(parameter) == null?false:whereParameters.get(parameter);
	}
	
	/**
	 * @param parameter
	 * @return boolean
	 */
	public Class<?> getParameterClass(String parameter) {
		if ( parameterClass == null ) {
			parameterClass = new HashMap<String,Class<?>>();
			if ( this.input.getMappings() != null ) {
				for ( InputMapping im : this.input.getMappings() ) {
					try {
						parameterClass.put(im.getSourceName(), this.getClass(im.getSourceClass()));
					} catch (ClassNotFoundException e) {
						logger.error("[Action:getParameterClass]", e);
					}
				}
			}
		}
		return parameterClass.get(parameter);
	}

	/**
	 * @return
	 */
	private boolean isOrmModel(){
		if ( !CommonTools.isNull(this.ormAction) ) {
			return true;
		}
		return false;
	}
	

	/**
	 * @return the ormAction
	 */
	public String getOrmAction() {
		return ormAction;
	}

	/**
	 * @param ormAction the ormAction to set
	 */
	public void setOrmAction(String ormAction) {
		this.ormAction = ormAction;
	}
	
}
