/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config.bean;

import com.nonesole.persistence2.config.CommonTools;

/**
 * @author jack lee
 * @version 2.0
 * @date 2016-09-29
 */
public class InputMapping {

	private String sourceName;
	private String sourceClass;
	private boolean where = false;
	private boolean notNull = false;
	private String sourceDefaultValue;
	private String sourceMinValue;
	private String sourceMaxValue;
	private int sourceMinLength;
	private int sourceMaxLength;
	private String sourceValueRule;
	private String targetName;
	private boolean isSqlPart = false;
	
	/**
	 * @return the isSqlPart
	 */
	public boolean isSqlPart() {
		return isSqlPart;
	}
	/**
	 * @param isSqlPart the isSqlPart to set
	 */
	public void setSqlPart(boolean isSqlPart) {
		this.isSqlPart = isSqlPart;
	}
	/**
	 * @return the sourceName
	 */
	public String getSourceName() {
		return sourceName;
	}
	/**
	 * @param sourceName the sourceName to set
	 */
	public void setSourceName(String sourceName) {
		this.sourceName = sourceName;
	}
	/**
	 * @return the sourceClass
	 */
	public String getSourceClass() {
		return sourceClass;
	}
	/**
	 * @param sourceClass the sourceClass to set
	 */
	public void setSourceClass(String sourceClass) {
		this.sourceClass = sourceClass;
	}
	/**
	 * @return the notNull
	 */
	public boolean isNotNull() {
		return notNull;
	}
	/**
	 * @param notNull the notNull to set
	 */
	public void setNotNull(boolean notNull) {
		this.notNull = notNull;
	}
	/**
	 * @return the sourceDefaultValue
	 */
	public String getSourceDefaultValue() {
		return sourceDefaultValue;
	}
	/**
	 * @param sourceDefaultValue the sourceDefaultValue to set
	 */
	public void setSourceDefaultValue(String sourceDefaultValue) {
		this.sourceDefaultValue = sourceDefaultValue;
	}
	/**
	 * @return the sourceMinValue
	 */
	public String getSourceMinValue() {
		return sourceMinValue;
	}
	/**
	 * @param sourceMinValue the sourceMinValue to set
	 */
	public void setSourceMinValue(String sourceMinValue) {
		this.sourceMinValue = sourceMinValue;
	}
	/**
	 * @return the sourceMaxValue
	 */
	public String getSourceMaxValue() {
		return sourceMaxValue;
	}
	/**
	 * @param sourceMaxValue the sourceMaxValue to set
	 */
	public void setSourceMaxValue(String sourceMaxValue) {
		this.sourceMaxValue = sourceMaxValue;
	}
	/**
	 * @return the sourceMinLength
	 */
	public int getSourceMinLength() {
		return sourceMinLength;
	}
	/**
	 * @param sourceMinLength the sourceMinLength to set
	 */
	public void setSourceMinLength(int sourceMinLength) {
		this.sourceMinLength = sourceMinLength;
	}
	/**
	 * @return the sourceMaxLength
	 */
	public int getSourceMaxLength() {
		return sourceMaxLength;
	}
	/**
	 * @param sourceMaxLength the sourceMaxLength to set
	 */
	public void setSourceMaxLength(int sourceMaxLength) {
		this.sourceMaxLength = sourceMaxLength;
	}
	/**
	 * @return the sourceValueRule
	 */
	public String getSourceValueRule() {
		return sourceValueRule;
	}
	/**
	 * @param sourceValueRule the sourceValueRule to set
	 */
	public void setSourceValueRule(String sourceValueRule) {
		this.sourceValueRule = sourceValueRule;
	}
	/**
	 * @return the targetName
	 */
	public String getTargetName() {
		return targetName;
	}
	/**
	 * @param targetName the targetName to set
	 */
	public void setTargetName(String targetName) {
		this.targetName = targetName;
	}
	/**
	 * @return
	 */
	public boolean sourceIsNumber(){
		if ( CommonTools.isNull(this.sourceClass) ) {
			return false;
		} else {
			if ( this.isDoubleValue() || this.isFloatValue() || this.isIntValue() 
					|| this.isLongValue() || this.isShortValue() ) {
				return true;
			} else {
				return false;
			}
		}
	}
	
	/**
	 * @param value
	 * @return
	 */
	public boolean isIntValue() {
		if ( "java.lang.Integer".equals(this.sourceClass) ||
				"int".equals(this.sourceClass) ){
			return true;
		}
		return false;
	}

	/**
	 * @param value
	 * @return
	 */
	public boolean isShortValue() {
		if ( "java.lang.Short".equals(this.sourceClass) ||
				"short".equals(this.sourceClass) ){
			return true;
		}
		return false;
	}
	/**
	 * @param value
	 * @return
	 */
	public boolean isLongValue() {
		if ( "java.lang.Long".equals(this.sourceClass) ||
				"long".equals(this.sourceClass) ){
			return true;
		}
		return false;
	}
	/**
	 * @param value
	 * @return
	 */
	public boolean isFloatValue() {
		if ( "java.lang.Float".equals(this.sourceClass) ||
				"float".equals(this.sourceClass) ){
			return true;
		}
		return false;
	}
	/**
	 * @param value
	 * @return
	 */
	public boolean isDoubleValue() {
		if ( "java.lang.Double".equals(this.sourceClass) ||
				"double".equals(this.sourceClass) ){
			return true;
		}
		return false;
	}
	/**
	 * @param value
	 * @return
	 */
	public boolean isEqual(Double d1,Double d2,String clazz) {
		if ( d1 == null || d2 == null || CommonTools.isNull(clazz)){
			return false;
		}
		if ( this.isShortValue() ) {
			return d1.shortValue()==d2.shortValue(); 
		} else if ( this.isIntValue() ) {
			return d1.intValue()==d2.intValue(); 
		} else if ( this.isLongValue() ) {
			return d1.longValue()==d2.longValue(); 
		} else if ( this.isFloatValue() ) {
			return Math.abs(d1.floatValue()-d2.floatValue())<0.01f; 
		} else if ( this.isDoubleValue() ) {
			return Math.abs(d1.doubleValue()-d2.doubleValue())<0.01d; 
		} 
		return false;
	}
	/**
	 * @return
	 */
	public boolean validate(){
		boolean validate = true;
		
		if ( !CommonTools.isNull(this.sourceDefaultValue) ) {
			if ( this.sourceIsNumber() ) {
				if ( !CommonTools.isNumber(this.sourceDefaultValue) ) {
					return false;
				} 
				Double defaultValue = Double.valueOf(this.sourceDefaultValue);
				validate = this.greaterThanMinValue(defaultValue);
				if ( !validate ) return validate;
				
				validate = this.lessThanMaxValue(defaultValue);
				if ( !validate ) return validate;
			} else {
				validate = this.matchValueRule(this.sourceDefaultValue);
				if ( !validate ) return validate;
				
				validate = this.greaterThanMinLength(this.sourceDefaultValue);
				if ( !validate ) return validate;
				
				validate = this.lessThanMaxLength(this.sourceDefaultValue);
			}
		}
		
		if ( CommonTools.isNull(this.targetName) ) {
			validate = false;
		}
		
		return validate;
	}
	
	/**
	 * @param value
	 * @return
	 */
	public boolean greaterThanMinValue(Double value) {
		boolean validate = false;
		if ( !CommonTools.isNull(this.sourceMinValue) ) {
			Double minValue = Double.valueOf(this.sourceMinValue);
			if ( this.isDoubleValue() ) {
				double r = value-minValue;
				validate = validate && (r>0) || Math.abs(r) < 0.01;
			} else if ( this.isFloatValue() ) {
				float r = value.floatValue()-minValue.floatValue();
				validate = validate &&  (r>0d) || Math.abs(r) < 0.01f;
			} else if ( this.isIntValue() ) {
				int r = value.intValue()-minValue.intValue();
				validate = validate &&  r>=0;
			} else if ( this.isLongValue() ) {
				long r = value.longValue()-minValue.longValue();
				validate = validate &&  r>=0l;
			} else if ( this.isShortValue() ) {
				int r = value.shortValue()-minValue.shortValue();
				validate = validate &&  r>=0;
			} 
		}
		return validate;
	}
	
	/**
	 * @param value
	 * @return
	 */
	public boolean lessThanMaxValue(Double value) {
		if ( value == null ) {
			return false;
		}
		boolean validate = false;
		if ( !CommonTools.isNull(this.sourceMaxValue) ) {
			Double maxValue = Double.valueOf(this.sourceMaxValue);
			if ( this.isDoubleValue() ) {
				double r = maxValue-value;
				validate = validate &&  (r>0) || Math.abs(r) < 0.01;
			} else if ( this.isFloatValue() ) {
				float r = maxValue.floatValue()-value.floatValue();
				validate = validate &&  (r>0d) || Math.abs(r) < 0.01f;
			} else if ( this.isIntValue() ) {
				int r = maxValue.intValue()-value.intValue();
				validate = validate &&  r>=0;
			} else if ( this.isLongValue() ) {
				long r = maxValue.longValue()-value.longValue();
				validate = validate &&  r>=0l;
			} else if ( this.isShortValue() ) {
				int r = maxValue.shortValue()-value.shortValue();
				validate = validate &&  r>=0;
			} 
		}
		return validate;
	}
	
	/**
	 * @return
	 */
	public boolean greaterThanMinLength(String value){
		if ( value == null ) {
			return false;
		}
		if ( this.sourceMinLength > 1 ) {//Ĭ��ֵ�ַ��Ȳ���С��Լ��ֵ
			if ( value.length() < this.sourceMinLength ) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * @return
	 */
	public boolean lessThanMaxLength(String value){
		if ( value == null ) {
			return false;
		}
		if ( this.sourceMaxLength > 1 ) {
			if ( value.length() > this.sourceMaxLength ) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * @return
	 */
	public boolean matchValueRule(String value){
		if ( value == null ) {
			return false;
		}
		if ( !CommonTools.isNull(this.sourceValueRule) ) {
			return value.matches(this.sourceValueRule);
		} 
		return false;
	}
	/**
	 * @return the where
	 */
	public boolean isWhere() {
		return where;
	}
	/**
	 * @param where the where to set
	 */
	public void setWhere(boolean where) {
		this.where = where;
	}
	
}
