/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config.bean;

import java.util.ArrayList;
import java.util.List;

import com.nonesole.persistence2.config.CommonTools;

/**
 * @author jack
 * @version 1.0
 */
public class OutputObject {

	public static final String VOID = "void";
	public static final String SIMPLE = "simple";
	public static final String POJO = "pojo";
	public static final String MAP = "map";
	public static final String SIMPLE_LIST = "simple-list";
	public static final String POJO_LIST = "pojo-list";
	public static final String MAP_LIST = "map-list";
	public static final String RESULT_SET = "resultset";
	
	private String property;
	private List<OutputMapping> mappings;
	private String objectClass;
	private String itemClass;
	
	public OutputObject(){
		mappings = new ArrayList<OutputMapping>();
	}
	/**
	 * @param mapping
	 */
	public void add(OutputMapping mapping) {
		if ( mapping != null )
			this.mappings.add(mapping);
	}
	/**
	 * @return the objectClass
	 */
	public String getObjectClass() {
		return objectClass;
	}

	/**
	 * @param objectClass the objectClass to set
	 */
	public void setObjectClass(String objectClass) {
		this.objectClass = objectClass;
	}

	/**
	 * @return the itemClass
	 */
	public String getItemClass() {
		return itemClass;
	}
	/**
	 * @param itemClass the itemClass to set
	 */
	public void setItemClass(String itemClass) {
		this.itemClass = itemClass;
	}
	public String getProperty() {
		return property;
	}
	public void setProperty(String property) {
		this.property = property;
	}
	public List<OutputMapping> getMappings() {
		return mappings;
	}
	public void setMappings(List<OutputMapping> mappings) {
		this.mappings = mappings;
	}
	/**
	 * @return
	 */
	public boolean validate(){
		boolean validate = true;
		if ( !CommonTools.isNull(this.property) && !this.mappings.isEmpty() ) {
			if ( POJO.equals(this.property) || 
					POJO_LIST.equals(this.property) ||
					VOID.equals(this.property) || 
					SIMPLE.equals(this.property) ||
					SIMPLE_LIST.equals(this.property) || 
					RESULT_SET.equals(this.property) ||
					MAP_LIST.equals(this.property) ||
					MAP.equals(this.property) ) {
				
				if ( POJO.equals(this.property) || 
						POJO_LIST.equals(this.property) || 
						RESULT_SET.equals(this.property) ||
							MAP_LIST.equals(this.property) ||
							MAP.equals(this.property) ){
					if ( !CommonTools.isNull(this.objectClass) ){
						if ( POJO_LIST.equals(this.property) || 
									MAP_LIST.equals(this.property) ){
							if ( CommonTools.isNull(this.itemClass) ) {
								return false;
							} 
						}
					} else {
						return false;
					}
				}
			} else {
				return false;
			}
		} else if ( CommonTools.isNull(this.property) && this.mappings.isEmpty() ) {
			return  true;
		} 
		
		return validate;
	}
		
}
