/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.connection.exception.OperationsException;

/**
 * Context of Connection Pool
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-08
 */
public class PoolContext {

	private static Context ctx;
	private static Context sysctx;
	
	private final static PoolContext cpc;
	
	static {
		cpc = new PoolContext();
		
		//initialize context,it's for preparing datasource
		Hashtable<String,Object> env = new Hashtable<String,Object>();
		env.put(Context.INITIAL_CONTEXT_FACTORY, 
				"com.nonesole.persistence.jndi.PersistenceInitCtxFactory");
		try {
			ctx = new InitialContext(env);
			sysctx = new InitialContext();
		} catch (NamingException e) {
			Logger.getInstance().error("There is an error on creating pool context.", e);
		}		
	}
	
	private PoolContext(){}
	
	/**
	 * Return ConnectionPool Context Object.
	 * @return ConnectionPoolContext
	 * */
	public synchronized static PoolContext getInstance(){
		return cpc;
	}
	
	/**
	 * Return the only one context of self.
	 * @return Context
	 * */
	public synchronized Context getDefaultContext(){
		return ctx;
	}
	
	/**
	 * Return the only one context of system.
	 * @return Context
	 * */
	public synchronized Context getSystemContext(){
		return sysctx;
	}
	
	/**
	 * Return datasource
	 * @param name datasource's name
	 * @return datasource
	 * @throws OperationsException 
	 * */
	public synchronized DataSource getDataSource(String name) 
			throws OperationsException {
		if ( null == name || "".equals(name) )
			return null;
		try {
			return (DataSource)ctx.lookup(name);
		} catch (NamingException e) {
			throw new OperationsException("Looking up datasource is failed.",e.fillInStackTrace());
		}
	}
	
}
