/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool.parser;

import java.util.Map;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.Constant;

/**
 * Tools of connection pool parser
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-20 
 */
public class ConnectionPoolParserUtil {

	/**
	 * Only for connection pool parser
	 * @param n Node object which include key,value property
	 * @param m result of parsing
	 * @param parameterFlag flag of Node
	 * */
	public final static void parseParameter(Node n,
			Map<String,Object> m,String parameterFlag) throws OperationsException{
		
		if ( null == n || null == m || null == parameterFlag ) {
			throw new OperationsException("Parameter is null,so parsing parameter is failed.");
		}
		
		NodeList l =  n.getChildNodes();
		for ( int i = 1 ; i < l.getLength() ; i++ ) {
			if ( "parameter".equals(l.item(i).getNodeName()) )
				parseParameter(l.item(i),m,Constant.PARAMETER_NAME,
						Constant.PARAMETER_VALUE);
		}
		
	}
	
	
	/**
	 * Only for connection pool parser
	 * @param n - Node object which include key,value property
	 * @param m - result of parsing
	 * @param keyFlag - flag of Node object and key flag of map
	 * @param valueFlag - flag of Node object and value flag of map
	 * */
	private final static void parseParameter(Node n,Map<String,Object> m,
									String keyFlag,String valueFlag) throws OperationsException{
		
		if ( null == n || null == m || null == keyFlag || null == valueFlag ) 
			throw new NullPointerException("Parameter is null,so parsing parameter is failed.");
		
		NodeList l = n.getChildNodes();
		String nodeName = null;
		int keyCount = 0;
		int valueCount = 0;
		String key = null;
		String value = null;
		for ( int i = 0 ; i < l.getLength() ; i++ ) {
			nodeName = l.item(i).getNodeName();
			if ( keyFlag.equals(nodeName) ) {//get key
				key = l.item(i).getTextContent();
				keyCount++;
			} else if ( valueFlag.equals(nodeName) ){//get value
				value = l.item(i).getTextContent();
				valueCount++;
			}
		}
		
		if ( keyCount != 1 && valueCount != 1 ) {
			throw new OperationsException("Searching parameter is failed." +
					"Cause the number of parameter is wrong.");
		} else if ( null == key || null == value  ) {
			throw new NullPointerException("Parameter is null,so parse is failed.");
		}
		//put the parameter which has been find into HashMap
		m.put(key, value);
		
	}
	
}
