/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool.parser;

/**
 * Properties of Connection Pool
 * @author JACK
 * @version 1.0 - build in 2009-07-01
 */
public class ConnectionPoolProperties {

	private String poolConfigName;
	private int maxStatements;
	private int initialPoolSize;
	private int minPoolSize;
	private int maxPoolSize;
	private int maxidleTime;
	private int propertyCycle;
		
	/**
	 * Current object is whether or not valid. 
	 * @return true - valid
	 * */
	public boolean isValid(){
		if ( initialPoolSize < minPoolSize 
				|| initialPoolSize > maxPoolSize
				|| minPoolSize > maxPoolSize
				|| initialPoolSize < 0
				|| minPoolSize < 0
				|| maxPoolSize < 0
				|| maxidleTime < 0
				|| propertyCycle < 0
				|| maxStatements < 0
				|| null == poolConfigName
				|| "".equals(poolConfigName) ) 
			return false;
		return true;
	}
	
	/**
	 * Rewrite toString method 
	 * @return String
	 * */
	public String toString(){
		return this.poolConfigName;
	}
	
	/**
	 * Rewrite equals method 
	 * this place do not need rewrite hashCode method.
	 * @return true or false
	 * */
	public boolean equals(Object obj){
		if ( obj instanceof ConnectionPoolProperties ) {
			ConnectionPoolProperties cpp = (ConnectionPoolProperties)obj;
			if ( this.poolConfigName.equals(cpp.getPoolConfigName()) 
					&& this.initialPoolSize == cpp.getInitialPoolSize()
					&& this.minPoolSize == cpp.getMinPoolSize()
					&& this.maxPoolSize == cpp.getMaxPoolSize()
					&& this.maxidleTime == cpp.getMaxidleTime()
					&& this.propertyCycle == cpp.getPropertyCycle()
					&& this.maxStatements == cpp.getMaxStatements() )
				return true;
		}
		return false;
	}
		
	/**
	 * @return the poolConfigName
	 */
	public String getPoolConfigName() {
		return poolConfigName;
	}

	/**
	 * @param poolConfigName the poolConfigName to set
	 */
	public void setPoolConfigName(String poolConfigName) {
		this.poolConfigName = poolConfigName;
	}
	/**
	 * @return the maxStatements
	 */
	public int getMaxStatements() {
		return maxStatements;
	}
	/**
	 * @param maxStatements the maxStatements to set
	 */
	public void setMaxStatements(int maxStatements) {
		this.maxStatements = maxStatements;
	}
	/**
	 * @return the initialPoolSize
	 */
	public int getInitialPoolSize() {
		return initialPoolSize;
	}
	/**
	 * @param initialPoolSize the initialPoolSize to set
	 */
	public void setInitialPoolSize(int initialPoolSize) {
		this.initialPoolSize = initialPoolSize;
	}
	/**
	 * @return the minPoolSize
	 */
	public int getMinPoolSize() {
		return minPoolSize;
	}
	/**
	 * @param minPoolSize the minPoolSize to set
	 */
	public void setMinPoolSize(int minPoolSize) {
		this.minPoolSize = minPoolSize;
	}
	/**
	 * @return the maxPoolSize
	 */
	public int getMaxPoolSize() {
		return maxPoolSize;
	}
	/**
	 * @param maxPoolSize the maxPoolSize to set
	 */
	public void setMaxPoolSize(int maxPoolSize) {
		this.maxPoolSize = maxPoolSize;
	}
	/**
	 * @return the maxidleTime
	 */
	public int getMaxidleTime() {
		return maxidleTime;
	}
	/**
	 * @param maxidleTime the maxidleTime to set
	 */
	public void setMaxidleTime(int maxidleTime) {
		this.maxidleTime = maxidleTime;
	}
	/**
	 * @return the propertyCycle
	 */
	public int getPropertyCycle() {
		return propertyCycle;
	}
	/**
	 * @param propertyCycle the propertyCycle to set
	 */
	public void setPropertyCycle(int propertyCycle) {
		this.propertyCycle = propertyCycle;
	}
}
