/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool.parser;

/**
 * JDBC driver
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-01
 */
public class DriverProperties {

	private String dataSourceName;
	private String driverClass;
	private String url;
	private String user;
	private String pwd;
	private boolean associateConnectionPool = false;//false => JDBC or true => ConnectionPool
	private String connectionPoolName;//if associateConnectionPool = false ,do not need this property
	
	/**
	 * Current object is whether or not valid. 
	 * @return true - valid
	 * */
	public boolean isValid(){
		if ( null == this.dataSourceName 
				|| null == this.driverClass
				|| null == this.url
				|| null == this.user
				|| null == this.pwd
				|| "".equals(this.dataSourceName)
				|| "".equals(this.driverClass)
				|| "".equals(this.url)
				|| "".equals(this.user) ) 
			return false;
		if ( associateConnectionPool && 
				( null == this.connectionPoolName
				|| "".equals(this.connectionPoolName)) )
			return false;
		return true;
	}
	
	/**
	 * Rewrite toString method 
	 * @return String
	 * */
	public String toString(){
		return this.dataSourceName;
	}
	
	/**
	 * Rewrite equals method 
	 * this place do not need rewrite hashCode method.
	 * @return true or false
	 * */
	public boolean equals(Object obj){
		if ( obj instanceof DriverProperties ) {
			DriverProperties dp = (DriverProperties)obj;
			if ( this.dataSourceName.equals(dp.getDataSourceName()) 
					&& this.driverClass.equals(dp.getDriverClass())
					&& this.url.equals(dp.getUrl())
					&& this.user.equals(dp.getUser())
					&& this.pwd.equals(dp.getPwd()) 
					&& this.associateConnectionPool == dp.isAssociateConnectionPool()
					&& ( this.associateConnectionPool == true 
							&& this.connectionPoolName.equals(dp.getConnectionPoolName()) ) )
				return true;
		}
		return false;
	}
	
	
	/**
	 * @return the associateConnectionPool
	 */
	public boolean isAssociateConnectionPool() {
		return associateConnectionPool;
	}
	/**
	 * @param associateConnectionPool the associateConnectionPool to set
	 */
	public void setAssociateConnectionPool(boolean associateConnectionPool) {
		this.associateConnectionPool = associateConnectionPool;
	}
	/**
	 * @return the connectionPoolName
	 */
	public String getConnectionPoolName() {
		return connectionPoolName;
	}
	/**
	 * @param connectionPoolName the connectionPoolName to set
	 */
	public void setConnectionPoolName(String connectionPoolName) {
		this.connectionPoolName = connectionPoolName;
	}
	/**
	 * @return the driverClass
	 */
	public String getDriverClass() {
		return driverClass;
	}
	/**
	 * @param driverClass the driverClass to set
	 */
	public void setDriverClass(String driverClass) {
		this.driverClass = driverClass;
	}
	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}
	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}
	/**
	 * @return the user
	 */
	public String getUser() {
		return user;
	}
	/**
	 * @param user the user to set
	 */
	public void setUser(String user) {
		this.user = user;
	}
	/**
	 * @return the pwd
	 */
	public String getPwd() {
		return pwd;
	}
	/**
	 * @param pwd the pwd to set
	 */
	public void setPwd(String pwd) {
		this.pwd = pwd;
	}
	/**
	 * @return the dataSourceName
	 */
	public String getDataSourceName() {
		return dataSourceName;
	}
	/**
	 * @param dataSourceName the dataSourceName to set
	 */
	public void setDataSourceName(String dataSourceName) {
		this.dataSourceName = dataSourceName;
	}
}
