/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.ConfigParser;
import com.nonesole.persistence2.config.XMLHandler;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.Actions;
import com.nonesole.persistence2.config.bean.OSMObject;
import com.nonesole.persistence2.sql.tools.FileHandler;

/**
 * The framework will automatically load the configuration 
 * through this class during system startup.
 * @author jack lee
 * @version 1.0
 */
public class ConfigLoader {

	private static final String CONFIG_FILE = "persistence2.xml";
	private static final String RESOURCE = "resource";
	private static final String RESOURCE_PATH = "path";
	private static ConfigLoader loader = null;
	private XMLHandler handler = null;
	private List<String> pathList = null;
	private List<OSMObject> osmList = null;
	private Map<String,OSMObject> osmMap = null;
	private Actions customAction = null;
	private Logger logger = null;
	
	
	static {
		loader = new ConfigLoader();
	}
	private ConfigLoader(){
		logger = Logger.getInstance();
		pathList = new ArrayList<String>();
		osmList = new ArrayList<OSMObject>();
		osmMap = new HashMap<String,OSMObject>();
		this.handler = XMLHandler.getInstance();
		this.loadXML();
		
	}
	
	/**
	 * Get singleton object 
	 * @return ConfigLoader
	 */
	public static ConfigLoader getInstance(){
		return loader;
	}
	
	/**
	 * Get the list of object-sql-mapping object
	 * @return List<ObjectSqlMapping>
	 */
	public List<OSMObject> getOSMList(){
		return this.osmList;
	}
	
	/**
	 * Get object-sql-mapping object
	 * @return ObjectSqlMapping
	 */
	public OSMObject getObjectSqlMapping(String className){
		return this.osmMap.get(className);
	}
	
	/**
	 * Get the action
	 * @return Action
	 */
	public Action getAction(String actionName){
		return this.customAction.get(actionName);
	}
		
	/**
	 * Load xml file of configuration
	 */
	private void loadXML(){
		try {
			Document doc = this.handler.loadStreamFile(CONFIG_FILE);
			if ( null != doc ) {
				List<Node> res = this.handler.getChildNodes(doc, RESOURCE);
				NamedNodeMap nnm = null;
				Node path = null;
				String value = null;
				for ( Node n : res ) {
					nnm = n.getAttributes();
					path = nnm.getNamedItem(RESOURCE_PATH);
					if ( null != path ) {
						value = path.getNodeValue();
						if ( null != value && !"".equals(value) ) {
							logger.debug(value, null);
							this.pathList.add(value);
						}
					}
				}
				if ( !this.pathList.isEmpty() ) {
					InputStream is = null;
					ConfigParser parser = null;
					for ( String p : this.pathList ) {
						logger.debug(p, null);
						is = FileHandler.getInstance().getResourceAsStream(p);
						parser = new ConfigParser(is);
						this.osmList.addAll(parser.osmParser());//����ORM����
						if ( null == this.customAction ) {
							this.customAction = new Actions();
						} 
						this.customAction.link(parser.actionsParser());
						is.close();
					}
					if ( null != this.osmList && !this.osmList.isEmpty() ) {
						for ( OSMObject osm : this.osmList ) {
							osmMap.put(osm.getClassName(), osm);
						}
					}
				}
			}
		} catch (Exception e) {
			Logger.getInstance().error("[ConfigLoader:loadXML]Can not parse the configration.", e);
		}
	}
	
}
