/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic;

import java.sql.Connection;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.sql.basic.input.InputTransform;
import com.nonesole.persistence2.sql.basic.output.OutputTransform;

/**
 * The basic interface of action which contains execute method and query method. 
 * @author jack lee
 * @version 1.0
 * @date 2016-10-29
 */
public interface IBasicAction {

	/**
	 * According to the rules in Action object which is loaded from xml file, <br>
	 * this method would call InputTransform object to assemble SQLs, and execute SQLs by itself.
	 * @param action
	 * @param conn
	 * @param partOfTransaction
	 * @param transform
	 * @throws Exception
	 */
	public void executeAction(Action action, Connection conn, boolean partOfTransaction, InputTransform transform) 
	throws Exception;
	
	/**
	 * According to the rules in Action object which is loaded from xml file, <br>
	 * this method would call OutputTransform object to assemble SQLs, and execute <br>
	 * SQLs by itself. In this method, the last SQL must be DQL and the ResultSet <br>
	 * will be transform to custom-formatted-data. <br>
	 * @param action
	 * @param conn
	 * @param inputTransform
	 * @param outTransform
	 * @throws Exception	
	 */
	public void queryAction(Action action, Connection conn, InputTransform inputTransform, OutputTransform<?> outTransform) 
	throws Exception;
	
	
}
