/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.input;

import java.util.List;

import com.nonesole.persistence2.config.bean.Action;

/**
 * The method used to execute SQL statement depends on <br>
 * SQLs in Action object being transformed by InputTransform.<br>
 * The types of parameters include simple-list, pojo, <br>
 * pojo-list, map, orm-pojo, orm-pojo-list. <br><br>
 * simple-list: List&lt;Integer&gt;, List&lt;Double&gt;, List&lt;String&gt;...<br>
 * pojo,orm-pojo: User, Role...<br>
 * pojo-list,orm-pojo-list: List&lt;User&gt;, List&lt;Role&gt;...<br>
 * map: Map<String, Object><br>
 * 
 * @author jack lee
 * @version 1.0
 * @date 2016-10-29
 */
public interface InputTransform {

	/**
	 * Assemble SQL by parameters, or transform object to SQL.
	 * @param action
	 * @throws Exception
	 */
	public void transform(Action action) throws Exception;
	
	/**
	 * @return the blobPositionMap
	 */
	public List<Integer> getBlobPositionMap(String preparedSQL);

	/**
	 * @return the valueMap
	 */
	public List<Object> getValueMap(String preparedSQL);

	/**
	 * @return the fieldTypeMap
	 */
	public List<Class<?>> getFieldTypeMap(String preparedSQL);
	
	/**
	 * If the SQL statement is known to be not prepared, 
	 * Users could call this method to get SQL statement.
	 * @return the SQLs
	 */
	public List<String> getSQLList();
	
}
