/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.input;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.ParserException;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.InputMapping;
import com.nonesole.persistence2.config.bean.InputObject;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;

/**
 * The subclass for map parameter of InputTransform
 * @author jack lee
 * @version 1.0
 * @date 2016-10-29
 */
public class MapInputTransform extends BasicInputTransform {

	private Map<String,Object> values = null;
	private boolean isPreparedStatement = true;
	
	/**
	 * Constructor
	 */
	public MapInputTransform(Map<String,Object> values){
		this(values,true);
	}
	
	/**
	 * Constructor
	 */
	public MapInputTransform(Map<String,Object> values, boolean isPreparedStatement){
		super();
		this.values = values;
		this.isPreparedStatement = isPreparedStatement;
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.basic.InputTransform#transform(com.nonesole.persistence2.config.bean.Action)
	 */
	@Override
	public void transform(Action action) throws Exception {
		super.transform(action);
		action.checkMapParam(values);
		if ( this.isPreparedStatement ) {
			this.preparedTransform(action);
		} else {
			this.statementTransform(action);
		}
	}
	
	/**
	 * Assemble prepared SQL statement
	 * @param action
	 * @throws Exception
	 */
	protected void preparedTransform(Action action) throws Exception {
		InputObject input = action.getInput();
		List<String> sqls = action.getSqls();
		String preparedSQL = null;
		List<Integer> blobPositionList = new ArrayList<Integer>();
		List<Object> valueList = null;
		List<Class<?>> fieldTypeList = null;
		if (input != null && input.getMappings() != null) {
			if (sqls != null) {
				List<InputMapping> list = input.getMappings();
				List<Integer> indexList = null;
				String sql = null;
				Map<Integer, Integer> indexMap = new HashMap<Integer, Integer>();
				Class<?> clazz = null;
				String partSql = null;
				Map<String,String> targetSourceNameMap = new HashMap<String,String>();
				Map<String,String> targetSourceClassMap = new HashMap<String,String>();
				List<String> targetNames = new ArrayList<String>();
				List<String> partSqlTargetNames = null;
				Map<Integer,String> valueClassesMap = new HashMap<Integer,String>();
				Map<Integer,String> valueKeysMap = new HashMap<Integer,String>();
				String tmpTargetName = null;
				for (int i = 0; i < sqls.size(); i++) {
					sql = sqls.get(i);
					indexList = new ArrayList<Integer>();
					preparedSQL = new String(sql);
					valueList = new ArrayList<Object>();
					fieldTypeList = new ArrayList<Class<?>>();
					for (InputMapping im : list) {
						if (!im.isSqlPart()) {
							targetNames.add(im.getTargetName());
							targetSourceNameMap.put(im.getTargetName(), im.getSourceName());
							targetSourceClassMap.put(im.getTargetName(), im.getSourceClass());
						} else {
							if (values.get(im.getTargetName()) != null) {
								partSql = (String) values.get(im.getTargetName());
								preparedSQL = preparedSQL.replace(im.getTargetName(), partSql);
								partSqlTargetNames = this.getTargetNameFromPartSQL(partSql);
								Object targetValue = null;
								for ( String tn : partSqlTargetNames ) {
									targetValue = values.get(tn);
									if ( targetValue == null ) {
										throw new Exception("There is an error in the config of '" + action.getName() + "' action.Please check the xml file.");
									}
									targetSourceNameMap.put(tn, tn);
									targetSourceClassMap.put(tn, targetValue.getClass().getName());
								}
								targetNames.addAll(partSqlTargetNames);
								
							} else {
								preparedSQL = preparedSQL.replace(
										im.getTargetName(), "");
							}
						}
					}
					for (int j = 0; j < targetNames.size(); j++) {
						int in = 0;
						tmpTargetName = targetNames.get(j);
						int length = in+tmpTargetName.length();
						while ( (in = preparedSQL.indexOf(tmpTargetName, in+length)) > -1 ) {
							if ( indexMap.get(in) == null ) {
								indexList.add(in);
								indexMap.put(in, in);
								valueClassesMap.put(in, targetSourceClassMap.get(tmpTargetName));
								valueKeysMap.put(in, targetSourceNameMap.get(tmpTargetName));
							}
						}
					}
					Collections.sort(indexList);
					
					for (int j = 0; j < targetNames.size(); j++) {
						int index = preparedSQL.indexOf(targetNames.get(j));
						if (index > -1) {
							preparedSQL = preparedSQL.replace(
									targetNames.get(j), "?");
						}
					}
					
					for (int objIndex = 0 ; objIndex < indexList.size() ; objIndex++) {
						valueList.add(values.get(valueKeysMap.get(indexList.get(objIndex))));
						clazz = DefaultBaseAction.str2Type(valueClassesMap.get(indexList.get(objIndex)));
						fieldTypeList.add(clazz);
					}
					this.blobPositionMap.put(preparedSQL, blobPositionList);
					this.fieldTypeMap.put(preparedSQL, fieldTypeList);
					this.valueMap.put(preparedSQL, valueList);
					this.SQLList.add(preparedSQL);
				}
			} else {
				throw new ParserException("The list of sql is empty.");
			}
		} else {
			if (sqls != null) {
				SQLList.addAll(sqls);
			} else {
				throw new ParserException("The list of sql is empty.");
			}
		}
	}
	
	/**
	 * Assemble not-prepared SQL statement
	 * @param action
	 */
	protected void statementTransform(Action action) throws Exception {
		InputObject input = action.getInput();
		List<String> sqls = action.getSqls();
		String returnSql = null;
		
		if (input != null && input.getMappings() != null) {
			if (sqls != null) {
				List<InputMapping> list = input.getMappings();
				String sql = null;
				Object value = null;
				for (int i = 0; i < sqls.size(); i++) {
					sql = sqls.get(i);
					returnSql = new String(sql);
					for (InputMapping im : list) {
						if (im.isSqlPart()) {
							throw new ParserException("InputMapping of "+ action.getName() + " is wrong, it's sql-part property is 'true'. ");
						} else {
							// ƴSQL
							value = values.get(im.getSourceName());
							if ( value == null) {
								if ( im.isIntValue() ) {
									returnSql = returnSql.replace(im.getTargetName(), "0");
								} else if ( im.isDoubleValue() ) {
									returnSql = returnSql.replace(im.getTargetName(), "0");
								} else if ( im.isFloatValue() ) {
									returnSql = returnSql.replace(im.getTargetName(), "0");
								} else if ( im.isLongValue() ) {
									returnSql = returnSql.replace(im.getTargetName(), "0");
								} else if ( im.isShortValue() ) {
									returnSql = returnSql.replace(im.getTargetName(), "0");
								} else {
									returnSql = returnSql.replace(im.getTargetName(), "''");
								}
							} else {
								returnSql = returnSql.replace(im.getTargetName(), String.valueOf(value));
							}

						}
					}
					SQLList.add(returnSql);
				}
			} else {
				throw new ParserException("The list of sql is empty.");
			}
		} else {
			if (sqls != null) {
				SQLList.addAll(sqls);
			} else {
				throw new ParserException("The list of sql is empty.");
			}
		}
	}
	
	/**
	 * Get custom variable '#value#' from SQL statement
	 * @param sql
	 * @return 
	 */
	public List<String> getTargetNameFromPartSQL(String sql){
		List<String> targetNames = new ArrayList<String>();
		if ( sql == null || "".equals(sql) ) {
			return targetNames;
		}
		int formIndex = 0;
		int beginIndex = -1;
		int endIndex = -1;
		while ( (beginIndex = sql.indexOf("#", formIndex)) > -1 ) {
			if ( (endIndex = sql.indexOf("#", beginIndex+1)) > -1 ) {
				targetNames.add(sql.substring(beginIndex, endIndex+1));
				formIndex = endIndex+1;
				beginIndex = -1;
				endIndex = -1;
			} else {
				break;
			}
		}
		return targetNames;
	}
}
