/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.input;

import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.bean.Action;

/**
 * The subclass for pojo parameter of InputTransform
 * @author jack lee
 * @version 1.0
 * @date 2016-10-30
 */
public class PojoInputTransform extends BasicInputTransform {

	private Object value = null;
	private MapInputTransform mt = null;

	/**
	 * Constructor
	 */
	public PojoInputTransform(Object value) {
		super();
		this.value = value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#transform(com.nonesole
	 * .persistence2.config.bean.Action)
	 */
	@Override
	public void transform(Action action) throws Exception {
		super.transform(action);
		Map<String,Object> valueMap = action.checkPojoParam(value);
		mt = new MapInputTransform(valueMap);
		mt.transform(action);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getSQLList()
	 */
	@Override
	public List<String> getSQLList() {
		return mt==null?this.SQLList:mt.SQLList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getBlobPositionMap
	 * (java.lang.String)
	 */
	@Override
	public List<Integer> getBlobPositionMap(String preparedSQL) {
		return mt==null?this.blobPositionMap.get(preparedSQL):mt.blobPositionMap.get(preparedSQL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getValueMap(java.lang
	 * .String)
	 */
	@Override
	public List<Object> getValueMap(String preparedSQL) {
		return mt==null?this.valueMap.get(preparedSQL):mt.valueMap.get(preparedSQL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getFieldTypeMap(java
	 * .lang.String)
	 */
	@Override
	public List<Class<?>> getFieldTypeMap(String preparedSQL) {
		return mt==null?this.fieldTypeMap.get(preparedSQL):mt.fieldTypeMap.get(preparedSQL);
	}
}
