/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.input;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.ParserException;
import com.nonesole.persistence2.config.bean.Action;

/**
 * The subclass for list of pojo parameter of InputTransform
 * @author jack lee
 * @version 1.0 
 * @date 2016-10-29
 */
public class PojoListInputTransform extends BasicInputTransform {
	
	private List<Object> values = null;
	private PojoInputTransform pojo = null;
	private Map<Integer,PojoInputTransform> map = null;
	private int objIndex = 0;
	private int objSQLs = 0;
	private int objSQLIndex = 0;
		
	/**
	 * Constructor
	 */
	public PojoListInputTransform(List<Object> values) {
		super();
		this.values = values;
		map = new HashMap<Integer,PojoInputTransform>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#transform(com.nonesole
	 * .persistence2.config.bean.Action)
	 */
	@Override
	public void transform(Action action) throws Exception {
		super.transform(action);
		if (CommonTools.isNull(values)) {
			throw new ParserException("Values are null.");
		}
		for ( int i = 0 ; i < values.size() ; i++ ) {
			pojo = new PojoInputTransform(values.get(i));
			map.put(i, pojo);
			if ( i == 0 ) {
				objSQLs = pojo.getSQLList().size();
			}
		}
		objIndex = 0;
		objSQLIndex = 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getSQLList()
	 */
	@Override
	public List<String> getSQLList() {
		return this.SQLList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getBlobPositionMap
	 * (java.lang.String)
	 */
	@Override
	public List<Integer> getBlobPositionMap(String preparedSQL) {
		return this.map.get(objIndex)==null?
				this.blobPositionMap.get(preparedSQL):this.map.get(objIndex).blobPositionMap.get(preparedSQL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getValueMap(java.lang
	 * .String)
	 */
	@Override
	public List<Object> getValueMap(String preparedSQL) {
		if ( objSQLIndex < objSQLs ) {
			objSQLIndex++;
		} else {
			objSQLIndex = 0;
			objIndex++;
		}
		return this.map.get(objIndex)==null?
				this.valueMap.get(preparedSQL):this.map.get(objIndex).getValueMap(preparedSQL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getFieldTypeMap(java
	 * .lang.String)
	 */
	@Override
	public List<Class<?>> getFieldTypeMap(String preparedSQL) {
		return this.map.get(objIndex)==null?
				this.fieldTypeMap.get(preparedSQL):this.map.get(objIndex).fieldTypeMap.get(preparedSQL);
	}
	
}
