/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.input;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.ParserException;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.InputMapping;
import com.nonesole.persistence2.config.bean.InputObject;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * The subclass for list of single-value parameter of InputTransform
 * @author jack lee
 * @version 1.0
 * @date 2016-10-30
 */
public class SimpleListInputTransform extends BasicInputTransform {

	private List<Object> values = null;
	private Map<Integer, Map<String,Object>> map = null;
	private int objIndex = 0;
	private int objSQLs = 0;
	private int objSQLIndex = 0;

	private static final String BLOB = "blob";
	private static final String TYPE = "type";
	private static final String VALUE = "value";
	
	
	/**
	 * Constructor
	 */
	public SimpleListInputTransform(List<Object> values) {
		super();
		this.values = values;
		map = new HashMap<Integer, Map<String,Object>>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#transform(com.nonesole
	 * .persistence2.config.bean.Action)
	 */
	@Override
	public void transform(Action action) throws Exception {
		super.transform(action);
		InputObject input = action.getInput();
		List<String> sqls = action.getSqls();
		if ( input == null ) {
			this.SQLList.addAll(sqls);
			return;
		}
		if (CommonTools.isNull(values)) {
			throw new ParserException("Values are null.");
		}
		
		objSQLs = sqls==null?0:sqls.size();
		Map<String,Object> objectMap = null;
		for (int i = 0; i < values.size(); i++) {
			objectMap = this.transformObject(input, sqls, values.get(i));
			map.put(i, objectMap);
		}
		objIndex = 0;
		objSQLIndex = 0;
	}
	
	/**
	 * Assemble prepared SQL statement
	 * @param input
	 * @param sqls
	 * @param value
	 * @return Map<String,Object>
	 * @throws ClassNotFoundException
	 * @throws ParserException
	 * @throws ExecuteSQLException 
	 */
	private Map<String,Object> transformObject(InputObject input, List<String> sqls, Object value) 
	throws ClassNotFoundException, ParserException, ExecuteSQLException{
		List<InputMapping> list = input.getMappings();
		if ( list == null || list.isEmpty() ) {
			throw new ParserException("Mapping is null.");
		}
		if ( list.size() != 1 ) {
			throw new ParserException("Mapping config is wrong.");
		}
		if ( sqls == null || sqls.isEmpty() ) {
			throw new ParserException("The list of sql is empty.");
		}
				
		String preparedSQL = null;
		List<Integer> blobPositionList = new ArrayList<Integer>();
		List<Object> valueList = new ArrayList<Object>();
		List<Class<?>> fieldTypeList = new ArrayList<Class<?>>();
		Map<String,Object> objMap = null;
		String sql = null; 
		InputMapping im = list.get(0);//List<Object>��inputֻ��һ��������
		Map<String, List<Integer>> blobPositionMap = new HashMap<String, List<Integer>>();
		Map<String, List<Object>> valueMap = new HashMap<String, List<Object>>();
		Map<String, List<Class<?>>> fieldTypeMap = new HashMap<String, List<Class<?>>>();
		for ( int i = 0 ; i < sqls.size() ; i++ ) {
			sql = sqls.get(i);
			preparedSQL = sql.replace(im.getTargetName(), "?");
			valueList.add(value);
			fieldTypeList.add(DefaultBaseAction.str2Type(im.getSourceClass()));
			objMap = new HashMap<String,Object>();
			
			blobPositionMap.put(preparedSQL, blobPositionList);
			fieldTypeMap.put(preparedSQL, fieldTypeList);
			valueMap.put(preparedSQL, valueList);
			
			this.SQLList.add(preparedSQL);
		}
		objMap.put(BLOB, blobPositionMap);
		objMap.put(TYPE, fieldTypeMap);
		objMap.put(VALUE, valueMap);
		return objMap;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getSQLList()
	 */
	@Override
	public List<String> getSQLList() {
		return this.SQLList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getBlobPositionMap
	 * (java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Integer> getBlobPositionMap(String preparedSQL) {
		Map<String,Object> mapObject = this.map.get(objIndex);
		if ( mapObject == null ) {
			return null;
		} else {
			Object blob = mapObject.get(BLOB);
			if ( blob != null ) {
				return ((Map<String, List<Integer>>)blob).get(preparedSQL);
			} else {
				return null;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getValueMap(java.lang
	 * .String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Object> getValueMap(String preparedSQL) {
		if (objSQLIndex < objSQLs) {
			objSQLIndex++;
		} else {
			objSQLIndex = 0;
			objIndex++;
		}
		Map<String,Object> mapObject = this.map.get(objIndex);
		if ( mapObject == null ) {
			return null;
		} else {
			Object value = mapObject.get(VALUE);
			if ( value != null ) {
				return ((Map<String, List<Object>>)value).get(preparedSQL);
			} else {
				return null;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.nonesole.persistence2.sql.basic.InputTransform#getFieldTypeMap(java
	 * .lang.String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Class<?>> getFieldTypeMap(String preparedSQL) {
		Map<String,Object> mapObject = this.map.get(objIndex);
		if ( mapObject == null ) {
			return null;
		} else {
			Object type = mapObject.get(TYPE);
			if ( type != null ) {
				return ((Map<String, List<Class<?>>>)type).get(preparedSQL);
			} else {
				return null;
			}
		}
	}

}
