/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.output;

import java.sql.ResultSet;

/**
 * If the caller object represents an SQL query returning a <br>
 * ResultSet object, it depends on ResultSet in Action object 
 * being transformed by OutputTransform. <br>
 * The types of return values include void, simple, simple-list, <br>
 * pojo, pojo-list, map-list. <br><br>
 * void: no return value.<br>
 * simple: int, long, String...<br>
 * simple-list: List&lt;Integer&gt;, List&lt;Long&gt;, List&lt;String&gt;...<br>
 * pojo: User, Role...<br>
 * pojo-list,orm-pojo-list: List&lt;User&gt;, List&lt;Role&gt;...<br>
 * map-list: List&lt;Map&lt;String, Object&gt;&gt;<br>
 * @author jack lee
 * @version 1.0
 * @date 2016-10-29
 */
public interface OutputTransform<T> {

	/**
	 * Transform ResultSet to Object
	 * @param rs
	 * @throws Exception
	 */
	public void transform(ResultSet rs) throws Exception;
	
	/**
	 * Get Object
	 * @return
	 * @throws Exception
	 */
	public T getObject() throws Exception;
	
}
