/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic.output;

import java.sql.ResultSet;
import java.util.List;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.OutputMapping;
import com.nonesole.persistence2.config.bean.OutputObject;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Transform ResultSet to POJO;
 * @author jack lee
 * @version 1.0
 * @date 2016-10-30
 */
public class PojoOutputTransform implements OutputTransform<Object> {
	
	private Object object = null;
	private Action action = null;
	
	public PojoOutputTransform(Action action){
		this.action = action;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.basic.output.OutputTransform#transform(java.sql.ResultSet)
	 */
	@Override
	public void transform(ResultSet rs) throws Exception {
		if ( rs == null ) {
			throw new ExecuteSQLException("ResultSet is null.");
		}
		if ( action == null ) {
			throw new ExecuteSQLException("Action is null.");
		}
		OutputObject output = action.getOutput();
		if ( output == null ) {
			throw new ExecuteSQLException("Output is null.");
		}
		if( rs.next() ) {
			if ( output.getObjectClass() == null ) {
				throw new ExecuteSQLException("Output class of " + action.getName() + " action is null, please check your config.");
			}
			Class<?> clazz = DefaultBaseAction.str2Type(output.getObjectClass());
			List<OutputMapping> mappings = output.getMappings();
			if ( mappings == null ) {
				throw new ExecuteSQLException("Output mappings is null.");
			}
			//transform ResultSet to object
			object = DefaultBaseAction.parseResultSetToObject(rs, clazz, mappings);
		}
	}

	@Override
	public Object getObject() throws Exception {
		return object;
	}

}
