/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.define;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * If the return value is dyadic list, this class should be used.
 * @author jack lee
 * @version 1.0
 */
public interface IDyadicListValueAction {

	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Short>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Short>> shortDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Short>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Short>> shortDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Short>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Short>> shortDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Integer>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Integer>> intDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Integer>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Integer>> intDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Integer>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Integer>> intDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Long>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Long>> longDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Long>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Long>> longDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Long>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Long>> longDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Float>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Float>> floatDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Float>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Float>> floatDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Float>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Float>> floatDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Double>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Double>> doubleDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Double>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Double>> doubleDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Double>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Double>> doubleDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Boolean>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Boolean>> booleanDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Boolean>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Boolean>> booleanDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Boolean>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Boolean>> booleanDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<String>>
	 * @throws ExecuteSQLException
	 */
	public List<List<String>> stringDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<String>>
	 * @throws ExecuteSQLException
	 */
	public List<List<String>> stringDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<String>>
	 * @throws ExecuteSQLException
	 */
	public List<List<String>> stringDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @return List<List<Object>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Object>> objDyadicListAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return List<List<Object>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Object>> objDyadicListAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * 
	 * This method should be used to assemble and execute SQLs 
	 * and transforms ResultSet to dyadic list .The parameters are 
	 * configuration of Action , and values in Map&lt;String, Object&gt;.
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return List<List<Object>>
	 * @throws ExecuteSQLException
	 */
	public List<List<Object>> objDyadicListAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
}
