/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.define;

import java.sql.Connection;
import java.util.List;

import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * This class is used to extends ORM interface. 
 * There are more DML and DQL methods could be chose.
 * @author jack lee
 * @version 1.0
 * @date 2016-10-05
 */
public interface IOrmPlusAction {

	/**
	 * Insert,update,delete one object to database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public void doAction(String actionName,Object obj) 
	throws ExecuteSQLException;
		
	/**
	 * Batch insert,update,delete objects to database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public void doBatchAction(String actionName,List<Object> list) 
	throws ExecuteSQLException;
	
	/**
	 * Select objects from database
	 * @param obj 
	 * @param conn
	 * @return List<Object>
	 * */
	public List<Object> doQueryAction(String actionName,Object obj)
	throws ExecuteSQLException;
	
	/**
	 * Insert,update,delete one object to database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public void doAction(String actionName,Object obj,Connection conn)
	throws ExecuteSQLException;
	
	/**
	 * Batch insert,update,delete objects to database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public void doBatchAction(String actionName,List<Object> list,Connection conn)
	throws ExecuteSQLException;
	
	/**
	 * Select objects from database
	 * @param obj 
	 * @param conn
	 * @return List<Object>
	 * */
	public List<Object> doQueryAction(String actionName,Object obj,Connection conn)
	throws ExecuteSQLException;
	
	/**
	 * Insert,update,delete one object to database
	 * @param obj 
	 * @param conn
	 * @param isPrepared
	 * @return object
	 * */
	public void doAction(String actionName,Object obj,Connection conn,boolean isPrepared)
	throws ExecuteSQLException;
	
	/**
	 * Batch insert,update,delete objects to database
	 * @param obj 
	 * @param conn
	 * @param isPrepared
	 * @return object
	 * */
	public void doBatchAction(String actionName,List<Object> list,Connection conn,boolean isPrepared)
	throws ExecuteSQLException;
	
	/**
	 * Select objects from database
	 * @param obj 
	 * @param conn
	 * @param isPrepared
	 * @return List<Object>
	 * */
	public List<Object> doQueryAction(String actionName,Object obj,Connection conn,boolean isPrepared)
	throws ExecuteSQLException;
	
}
