/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.define;

import java.sql.Connection;
import java.util.Map;

import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * If the return value is single value, this class should be used.
 * @author jack lee
 * @version 1.0
 */
public interface ISingleValueAction {


	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input 
	 * @return Short
	 * @throws ExecuteSQLException
	 */
	public Short shortAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return Short
	 * @throws ExecuteSQLException
	 */
	public Short shortAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return Short
	 * @throws ExecuteSQLException
	 */
	public Short shortAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input
	 * @return Integer
	 * @throws ExecuteSQLException
	 */
	public Integer intAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input
	 * @param conn 
	 * @return Object
	 * @throws ExecuteSQLException
	 */
	public Integer intAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return Integer
	 * @throws ExecuteSQLException
	 */
	public Integer intAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @return Long
	 * @throws ExecuteSQLException
	 */
	public Long longAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return Long
	 * @throws ExecuteSQLException
	 */
	public Long longAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return Long
	 * @throws ExecuteSQLException
	 */
	public Long longAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @return Float
	 * @throws ExecuteSQLException
	 */
	public Float floatAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input
	 * @param conn 
	 * @return Float
	 * @throws ExecuteSQLException
	 */
	public Float floatAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn
	 * @param isPrepared 
	 * @return Float
	 * @throws ExecuteSQLException
	 */
	public Float floatAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @return Double
	 * @throws ExecuteSQLException
	 */
	public Double doubleAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @return Double
	 * @throws ExecuteSQLException
	 */
	public Double doubleAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input
	 * @param conn 
	 * @param isPrepared 
	 * @return Double
	 * @throws ExecuteSQLException
	 */
	public Double doubleAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @return Boolean
	 * @throws ExecuteSQLException
	 */
	public Boolean booleanAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input 
	 * @param conn 
	 * @return Boolean
	 * @throws ExecuteSQLException
	 */
	public Boolean booleanAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @param conn 
	 * @param isPrepared 
	 * @return Boolean
	 * @throws ExecuteSQLException
	 */
	public Boolean booleanAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input 
	 * @return String
	 * @throws ExecuteSQLException
	 */
	public String stringAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input
	 * @param conn
	 * @return String
	 * @throws ExecuteSQLException
	 */
	public String stringAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName 
	 * @param input
	 * @param conn
	 * @param isPrepared
	 * @return String
	 * @throws ExecuteSQLException
	 */
	public String stringAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input
	 * @return Object
	 * @throws ExecuteSQLException
	 */
	public Object objAction(String actionName, Map<String,Object> input) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input
	 * @param conn
	 * @return Object
	 * @throws ExecuteSQLException
	 */
	public Object objAction(String actionName, Map<String,Object> input, Connection conn) 
			throws ExecuteSQLException;
	
	/**
	 * Execute action and return value
	 * @param actionName
	 * @param input
	 * @param conn
	 * @param isPrepared
	 * @return Object
	 * @throws ExecuteSQLException
	 */
	public Object objAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
			throws ExecuteSQLException;
	
}
