/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.tools;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.sql.ResultSet;
import java.util.Map;

import com.nonesole.persistence2.connection.exception.OperationsException;

/**
 * Reflect Tools
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-01
 */
public class ReflectToolkit {

	/**
	 * Transform Map to JavaBean
	 * @param map Map which has fields and values
	 * @param c Class of JavaBean
	 * @return JavaBean object
	 * */
	public final static Object parseMapToObject(Map<String,Object> map,Class<?> c) throws Exception{
		if ( null == map )
			return null;
		
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new Exception("Fields is null.");
		
		Object obj = c.newInstance();
		String name;//field name
		Object value;//field value
		Class<?> type = null;
		for( Field field : fields ){
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				continue;
			} 
			name = field.getName();
			value = map.get(field.getName());
			type = field.getType();
			setFieldValue(name, type, obj,value);
		}
		
		return obj;
	}
	
	/**
	 * Transform ResultSet to JavaBean
	 * @param rs ResultSet object
	 * @param c Class of JavaBean
	 * @return JavaBean object
	 * */
	public final static void parseResultSetToObject(ResultSet rs,Object obj) throws Exception{
		if ( null == rs )
			throw new Exception("ResultSet is null.");
		if ( null == obj )
			throw new Exception("Object is null.");
		
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new Exception("Fields is null.");
		
		String name;//field name
		Object value;//field value
		Class<?> type = null;
		for( Field field : fields ){
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				continue;
			} 
			name = field.getName();
			type = field.getType();
			value = rs.getObject(name);
			setFieldValue(name, type, obj,value);
		}
		
	}
	
	/**
	 * Get value by get method
	 * @param field Field object
	 * @param obj owner of field
	 * @throws OperationsException
	 * */
	public static Object getFieldValue(
			Field field,Object obj) 
	throws OperationsException{
		Method m = null;
		
		int modifier = field.getModifiers();
		if ( Modifier.isFinal(modifier) ||
				(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
			return null;
		} 
		String fieldName = field.getName();
		Class<?> fieldType = field.getType();
		Class<?> classType = obj.getClass();
		StringBuilder sb = new StringBuilder();
		if ( fieldType == boolean.class ) {
		    if ( "is".equals(fieldName.substring(0,2)) )
		    	sb.append("is").
				append(fieldName.substring(2, fieldName.length()));
		    else
				sb.append("is").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		
		} else
			sb.append("get").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		try {
			m = classType.getMethod(sb.toString(),new Class[]{});
		} catch (SecurityException e) {
			throw new OperationsException(
					"Getting method failure.",e.fillInStackTrace());
		} catch (NoSuchMethodException e) {
			throw new OperationsException(
					"Getting method failure.",e.fillInStackTrace());
		}
		try {
			return m.invoke(obj, new Object[]{});
		} catch (IllegalArgumentException e) {
			throw new OperationsException(
					"Invoking method failure.",e.fillInStackTrace());
		} catch (IllegalAccessException e) {
			throw new OperationsException(
					"Invoking method failure.",e.fillInStackTrace());
		} catch (InvocationTargetException e) {
			throw new OperationsException(
					"Invoking method failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Set field value by set method of object
	 * @param fieldName field name
	 * @param fieldType field type
	 * @param obj owner of field
	 * @param parameter value of filed
	 * @throws OperationsException
	 * */
	public static void setFieldValue(
			String fieldName,Class<?> fieldType,Object obj,Object parameter) 
	throws OperationsException{
		Class<?> clazz = obj.getClass();
		Method m = null;
		if ( null != parameter ) {
			StringBuilder sb = new StringBuilder();
			if ( fieldName.matches("is[A-Z]+[\\w|\\W]*") ) {
				fieldName = fieldName.replace("is", "");
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			} else
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			try {
				m = clazz.getMethod(sb.toString(),fieldType);
			} catch (SecurityException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (NoSuchMethodException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			}
			//Transform string to int etc.
			if ( String.class == parameter.getClass() && String.class != fieldType ) {
				String v = parameter.toString().trim();
				if ( boolean.class == fieldType || Boolean.class == fieldType ) {
					parameter = Boolean.parseBoolean(v);
				} else if ( int.class == fieldType || Integer.class == fieldType ) {
					parameter = Integer.valueOf(v).intValue();
				} else if ( long.class == fieldType || Long.class == fieldType ) {
					parameter = Long.valueOf(v).longValue();
				} else if ( float.class == fieldType || Float.class == fieldType ) {
					parameter = Float.valueOf(v).floatValue();
				} else if ( double.class == fieldType || Double.class == fieldType ) {
					parameter = Double.valueOf(v).doubleValue();
				} else if ( short.class == fieldType || Short.class == fieldType ) {
					parameter = Short.valueOf(v).shortValue();
				} else if ( Byte.class == fieldType || byte.class == fieldType ) {
					parameter = Byte.valueOf(v).byteValue();
				} 
			}
			try {
				m.invoke(obj, new Object[]{parameter});
			} catch (IllegalArgumentException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (IllegalAccessException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (InvocationTargetException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			}			
		}	
	}
	
}
